/*******************************************************************\
| CLT_Main.CC                                                       |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

//-------------------------------------------------------------------

#define CLT_MaxLineLen          1024

//-------------------------------------------------------------------

// Do not use CORBA var classes as global variables, it causes
// problems with proper initialization and shutdown order.

Server_ptr		pMEAServer;

//-------------------------------------------------------------------

int main (int iArgC, char *apArgV[])
{
  CORBA::Object_ptr     pReference;

  // The initialization proceeds in a specific order. First, the
  // libraries are initialized. Second, the system information
  // routines are initialized. Third, the broker is initialized.

  TIMInit ();
  RNDInit (TIMGetTimestamp ());

  MEMInit ();
  NETInit ();
  PROInit ();

  ORBInitClient (iArgC, apArgV);

  // The benchmark is contained within other modules.

  ORBEnvVar
  ORBExcTry
  {
    #if   defined DSPFormatRAW

    #elif defined DSPFormatTXT

      // Display header

      DSPStdOut << DSP_HeaderBenchie
                << "[STR]\tVersion\t" << SAM_BenchmarkName " " SAM_GlobalVersion " " SAM_GlobalDate << '\t'
                << "[STR]\tSuite\t" << BENGetName () << '\t'
                << DSPEndLine;

      // Display configuration

      DSPStdOut << DSP_HeaderConfig
                << "[1]\tScale\t" << iTIMTimerScale << '\t'
                << "[1]\tGranularity\t" << iTIMTimerGranularity << '\t'
                << DSPEndLine;

    #else
      #error
    #endif

    // Execute measurement

    LOCLocateServer (pMEAServer);

    pReference = pMEAServer->Reference (ORBArgOnly); ORBExcCheck
    BENClient (pReference, iArgC, apArgV);

    pMEAServer->Terminate (ORBArgOnly); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("CLT", "Main " << ORBExcShow)
  };

  ORBDoneClient ();

  return (0);
};

//-------------------------------------------------------------------

