/*******************************************************************\
| ORB_OrbRiver_2.5.X_CC                                             |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

//-------------------------------------------------------------------

// Do not use CORBA var classes as global variables, it causes
// problems with proper initialization and shutdown order.

CORBA::ORB_ptr                  pORB;

PortableServer::POA_ptr         pRootPOA;
PortableServer::POAManager_ptr	pRootPOAManager;

//-------------------------------------------------------------------

#define ORBSwitchPrefix         "-ORB"

//-------------------------------------------------------------------

/*******************************************************************\
| ORBInitClient                                                     |
\*******************************************************************/

void ORBInitClient (int &iArgC, char *apArgV[])
{
  char **apOldArgV;
  char **apNewArgV;

  ORBEnvVar
  ORBExcTry
  {
    pORB = CORBA::ORB_init (iArgC, apArgV, "" ORBArgNext); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("ORB", "InitClient " << ORBExcShow)
  };

  // The arguments are not removed from the command line when
  // the broker processes them, we have to do it manually
  // afterwards.

  // Because the exact format of the arguments is not known,
  // all pairs of arguments where first starts with -ORB
  // are removed.

  apOldArgV = apArgV;
  apNewArgV = apArgV;

  for (int i = iArgC ; i > 0 ; )
  {
    if (strncmp (*apOldArgV, ORBSwitchPrefix, sizeof (ORBSwitchPrefix) - 1))
    {
      *apNewArgV = *apOldArgV;
      apOldArgV ++;
      apNewArgV ++;
      i         --;
    }
    else
    {
      apOldArgV += 2;
      iArgC     -= 2;
      i         -= 2;
    }
  }
}

/*******************************************************************\
| ORBInitServer                                                     |
\*******************************************************************/

void ORBInitServer (int &iArgC, char *apArgV[])
{
  CORBA::Object_var vRootPOABase;

  ORBInitClient (iArgC, apArgV);

  ORBEnvVar
  ORBExcTry
  {
    vRootPOABase = pORB->resolve_initial_references ("RootPOA" ORBArgNext); ORBExcCheck
    pRootPOA = PortableServer::POA::_narrow (vRootPOABase.in ());

    pRootPOAManager = pRootPOA->the_POAManager (ORBArgOnly); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("ORB", "InitServer " << ORBExcShow)
  }
}

/*******************************************************************\
| ORBDoneClient                                                     |
\*******************************************************************/

void ORBDoneClient (void)
{
  ORBEnvVar
  ORBExcTry
  {
    pORB->destroy (ORBArgOnly); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("ORB", "DoneClient " << ORBExcShow)
  }
}

/*******************************************************************\
| ORBDoneServer                                                     |
\*******************************************************************/

void ORBDoneServer (void)
{
  ORBDoneClient ();
}

//-------------------------------------------------------------------

