/*******************************************************************\
| ORB_TAO_1.2.X.CC                                                  |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

//-------------------------------------------------------------------

#define ORB_ThreadPoolSize      10

// Do not use CORBA var classes as global variables, it causes
// problems with proper initialization and shutdown order.

CORBA::ORB_ptr                  pORB;

PortableServer::POA_ptr         pRootPOA;
PortableServer::POAManager_ptr	pRootPOAManager;

THRThread                       asThreadPool [ORB_ThreadPoolSize - 1];

//-------------------------------------------------------------------

/*******************************************************************\
| ThreadListen                                                      |
\*******************************************************************/

void *ThreadListen (void *pArgs)
{
  ORBEnvVar
  ORBExcTry
  {
    pORB->run (ORBArgOnly); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("ORB", "ORBinit " << ORBExcShow)
  }

  return (NULL);
}

//-------------------------------------------------------------------

/*******************************************************************\
| ORBInitClient                                                     |
\*******************************************************************/

void ORBInitClient (int &iArgC, char *apArgV[])
{
  ORBEnvVar
  ORBExcTry
  {
    pORB = CORBA::ORB_init (iArgC, apArgV, "" ORBArgNext); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("ORB", "InitClient " << ORBExcShow)
  }
}

/*******************************************************************\
| ORBInitServer                                                     |
\*******************************************************************/

void ORBInitServer (int &iArgC, char *apArgV[])
{
  CORBA::Object_var vRootPOABase;
  uint32 iThread;

  ORBInitClient (iArgC, apArgV);

  ORBEnvVar
  ORBExcTry
  {
    vRootPOABase = pORB->resolve_initial_references ("RootPOA" ORBArgNext); ORBExcCheck
    pRootPOA = PortableServer::POA::_narrow (vRootPOABase.in ());

    pRootPOAManager = pRootPOA->the_POAManager (ORBArgOnly); ORBExcCheck

    // We want to make TAO use the thread pool model, which
    // requires launching extra threads that block inside
    // the listening loop.

    for (iThread = 0 ; iThread < sizeofarray (asThreadPool) ; iThread ++)
      THRThreadCreate (&asThreadPool [iThread], ThreadListen, NULL);
  }
  ORBExcCatch
  {
    DSPFatal ("ORB", "InitServer " << ORBExcShow)
  }
}

/*******************************************************************\
| ORBDoneClient                                                     |
\*******************************************************************/

void ORBDoneClient (void)
{
  ORBEnvVar
  ORBExcTry
  {
    pORB->destroy (ORBArgOnly); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("ORB", "DoneClient " << ORBExcShow)
  }
}

/*******************************************************************\
| ORBDoneServer                                                     |
\*******************************************************************/

void ORBDoneServer (void)
{
  uint32 iThread;

  ORBEnvVar
  ORBExcTry
  {
    // Wait for the extra threads that block inside the listening loop.

    for (iThread = 0 ; iThread < sizeofarray (asThreadPool) ; iThread ++)
      THRThreadFinish (&asThreadPool [iThread]);
  }
  ORBExcCatch
  {
    DSPFatal ("ORB", "DoneServer " << ORBExcShow)
  }

  ORBDoneClient ();
}

//-------------------------------------------------------------------
