/*******************************************************************\
| NET_Linux_Proc.CC                                                 |
+-------------------------------------------------------------------+
| The network information functions for Linux.                      |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

//-------------------------------------------------------------------

#define NET_MaxLineLen  1024

#define NET_MaxInterfaces       10

uint64 aiSentBytes              [NET_MaxInterfaces];
uint64 aiReceivedBytes          [NET_MaxInterfaces];
uint64 aiSentPackets            [NET_MaxInterfaces];
uint64 aiReceivedPackets        [NET_MaxInterfaces];

//-------------------------------------------------------------------

/*******************************************************************\
| NETInit                                                           |
\*******************************************************************/

void NETInit (void)
{
  uint32 i;

  for (i = 0 ; i < NET_MaxInterfaces ; i ++)
  {
    aiSentBytes       [i] = 0;
    aiReceivedBytes   [i] = 0;
    aiSentPackets     [i] = 0;
    aiReceivedPackets [i] = 0;
  }
}

//-------------------------------------------------------------------

/*******************************************************************\
| NETGetTotalTraffic                                                |
\*******************************************************************/

void NETGetTotalTraffic (uint64 *pSentBytes, uint64 *pReceivedBytes,
                         uint64 *pSentPackets, uint64 *pReceivedPackets)
{
  FILE                  *pFile;
  char                  acLine [NET_MaxLineLen];
  void                  *pDummy;
  int                   iDummy;
  uint32                iInterface;
  uint32                iInterfaceSentBytes, iInterfaceReceivedBytes;
  uint32                iInterfaceSentPackets, iInterfaceReceivedPackets;

  (*pSentBytes)   = (*pReceivedBytes)   = 0;
  (*pSentPackets) = (*pReceivedPackets) = 0;

  pFile = fopen ("/proc/net/dev", "r");
  assert (pFile != NULL);
  pDummy = fgets (acLine, sizeof (acLine), pFile);
  assert (pDummy != NULL);
  pDummy = fgets (acLine, sizeof (acLine), pFile);
  assert (pDummy != NULL);

  // The benchmarks run long enough to cause the counters to
  // overflow occasionally, which is why we attempt to
  // detect and compensate for that situation.

  iInterface = 0;
  while (fgets (acLine, sizeof (acLine), pFile) != NULL)
  {
    assert (iInterface < NET_MaxInterfaces);

    sscanf (acLine, " %*[^:]: %lu %lu %*u %*u %*u %*u %*u %*u %lu %lu",
            &iInterfaceReceivedBytes, &iInterfaceReceivedPackets,
            &iInterfaceSentBytes, &iInterfaceSentPackets);

    #define UpdateValue(Data)                           \
    {                                                   \
      ai##Data [iInterface] +=                          \
        iInterface##Data -                              \
        (uint32) ai##Data [iInterface];                 \
      (*p##Data) +=                                     \
        ai##Data [iInterface];                          \
    }

    UpdateValue (SentBytes);
    UpdateValue (ReceivedBytes);
    UpdateValue (SentPackets);
    UpdateValue (ReceivedPackets);

    #undef UpdateValue

    iInterface ++;
  }

  iDummy = fclose (pFile);
  assert (!iDummy);
}

//-------------------------------------------------------------------

