/*******************************************************************\
| PRO_Linux_Proc.CC                                                 |
+-------------------------------------------------------------------+
| The processor information functions for Linux.                    |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

//-------------------------------------------------------------------

#define PRO_MaxLineLen  1024

//-------------------------------------------------------------------

/*******************************************************************\
| PROInit                                                           |
\*******************************************************************/

void PROInit (void) { }

//-------------------------------------------------------------------

/*******************************************************************\
| PROGetApplicationThreads                                          |
\*******************************************************************/

void PROGetApplicationThreads (uint64 *pThreads)
{
  FILE                  *pFile;
  char                  acLine [PRO_MaxLineLen];
  int                   iDummy;

  pFile = fopen ("/proc/self/status", "r");
  assert (pFile != NULL);
  while (fgets (acLine, sizeof (acLine), pFile) != NULL)
  {
    sscanf (acLine, "Threads: %qu", pThreads);
  }
  iDummy = fclose (pFile);
  assert (!iDummy);
}

//-------------------------------------------------------------------

/*******************************************************************\
| PROGetTotalUsage                                                  |
\*******************************************************************/

void PROGetTotalUsage (uint64 *pKernelTicks,
                       uint64 *pWaitTicks,
                       uint64 *pUserTicks,
                       uint64 *pIdleTicks)
{
  FILE                  *pFile;
  char                  acLine [PRO_MaxLineLen];
  int                   iDummy;
  uint64                iUserJiffies;
  uint64                iNiceJiffies;
  uint64                iKernelJiffies;
  uint64                iIdleJiffies;

  pFile = fopen ("/proc/stat", "r");
  assert (pFile != NULL);
  while (fgets (acLine, sizeof (acLine), pFile) != NULL)
  {
    sscanf (acLine, "cpu %qu %qu %qu %qu", &iUserJiffies, &iNiceJiffies, &iKernelJiffies, &iIdleJiffies);
  }
  iDummy = fclose (pFile);
  assert (!iDummy);

  (*pKernelTicks) = iKernelJiffies;
  (*pWaitTicks) = 0;
  (*pUserTicks) = iUserJiffies + iNiceJiffies;
  (*pIdleTicks) = iIdleJiffies;
}

//-------------------------------------------------------------------

