/*******************************************************************\
| PLT_Windows_PData.H                                               |
+-------------------------------------------------------------------+
| Platform specific function to access Windows Performance Data.    |
+-------------------------------------------------------------------+
| Copyright (C) Lubomir Bulej <bulej@nenya.ms.mff.cuni.cz>          |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#ifndef __PLT_Windows_PData
#define __PLT_Windows_PData

//-------------------------------------------------------------------

#include <tchar.h>
#include <windows.h>
#include <winperf.h>

//-------------------------------------------------------------------

#define PLT_IndexDigits     32
#define PLT_IndexFormat     TEXT("%u")
#define PLT_MaxIterations   10

//-------------------------------------------------------------------

extern DWORD iPerfDataLen;
extern LPVOID pPerfDataMem;

//-------------------------------------------------------------------

/*******************************************************************\
| Performance Data Navigation                                       |
\*******************************************************************/

PERF_OBJECT_TYPE *         PLTFirstObjType (PERF_DATA_BLOCK * pDataBlk);
PERF_OBJECT_TYPE *         PLTNextObjType  (PERF_OBJECT_TYPE * pObjType);

PERF_INSTANCE_DEFINITION * PLTFirstInstDef (PERF_OBJECT_TYPE * pObjType);
PERF_INSTANCE_DEFINITION * PLTNextInstDef  (PERF_INSTANCE_DEFINITION * pInstDef);

PERF_COUNTER_DEFINITION *  PLTFirstCntrDef (PERF_OBJECT_TYPE * pObjType);
PERF_COUNTER_DEFINITION *  PLTNextCntrDef  (PERF_COUNTER_DEFINITION * pCntrDef);

PERF_COUNTER_BLOCK *       PLTFirstCntrBlk (PERF_OBJECT_TYPE * pObjType);
PERF_COUNTER_BLOCK *       PLTNextCntrBlk  (PERF_COUNTER_BLOCK * pCntrBlk);
PERF_COUNTER_BLOCK *       PLTInstCntrBlk  (PERF_INSTANCE_DEFINITION * pInstDef);

/*******************************************************************\
| Performance Data Lookup                                           |
\*******************************************************************/

DWORD                     PLTFindPerfIndex   (const TCHAR * pName);
PERF_OBJECT_TYPE *        PLTFindPerfObject  (PERF_DATA_BLOCK * pDataBlk, DWORD iIndex);
PERF_COUNTER_DEFINITION * PLTFindPerfCounter (PERF_OBJECT_TYPE * pObjType, DWORD iIndex);

/*******************************************************************\
| Performance Data Retrieval                                        |
\*******************************************************************/

PERF_DATA_BLOCK * PLTGetPerfData    (TCHAR * pValue, LPVOID * ppDataPtr, DWORD * piDataLen);
UINT64         PLTGetPerfCounter (PERF_COUNTER_BLOCK * pCntrBlk, PERF_COUNTER_DEFINITION * pCntrDef);

/*******************************************************************\
| Thread Local Memory Retrieval                                     |
\*******************************************************************/

VOID PLTGetPerfMemory (LPVOID * ppMemoryPtr, DWORD * piMemoryLen);
VOID PLTPutPerfMemory (LPVOID pMemoryPtr, DWORD iMemoryLen);
VOID PLTDelPerfMemory (VOID);

/*******************************************************************\
| Performance Data Mutex                                            |
\*******************************************************************/

VOID PLTAcquirePerfMutex (VOID);
VOID PLTReleasePerfMutex (VOID);

/*******************************************************************\
| Platform Library Initialization                                   |
\*******************************************************************/

VOID PLTInitialize (VOID);

//-------------------------------------------------------------------

#endif // __PLT_Windows_PData
