/*******************************************************************\
| RND_Posix_Random.CC                                               |
+-------------------------------------------------------------------+
| Random number generator based on random and srandom.              |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

//-------------------------------------------------------------------

uint32    RNDRandomMask;
int       RNDRandomBits;

//-------------------------------------------------------------------

/*******************************************************************\
| RNDInit                                                           |
\*******************************************************************/

void RNDInit (uint64 iSeed)
{
  // Initialization of the random number generator, just old srandom()
  // call with a seed. Besides that, we also calculate the number and
  // mask of the bits we can take seriously in the random library call
  // return value based on the RAND_MAX constant.

  uint64 iRandom;

  iRandom = ((((uint64) RAND_MAX) - 1) >> 1) + 1;
  RNDRandomMask = 0;
  RNDRandomBits = 0;
  while (iRandom)
  {
    RNDRandomMask |= iRandom;
    RNDRandomBits ++;
    iRandom >>= 1;
  };

  srandom (iSeed);
};

/*******************************************************************\
| RNDGetRandom                                                      |
\*******************************************************************/

uint64 RNDGetRandom (uint64 iModulo)
{
  // Based on the number of random bits we know the random library call to
  // return, we combine together a number of the calls to create the random
  // value to return.

  uint64        iResult;
  int           iBits;

  iBits = sizeof (iResult) * 8;
  iResult = 0;

  while (iBits > 0)
  {
    iResult <<= RNDRandomBits;
    iResult |= random () & RNDRandomMask;
    iBits   -= RNDRandomBits;
  };

  return (iModulo ? iResult % iModulo : iResult);
};

//-------------------------------------------------------------------
