/*******************************************************************\
| THR_Posix_Threads.CC                                              |
+-------------------------------------------------------------------+
| The thread wrappers for Posix threads.                            |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include <unistd.h>

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

//-------------------------------------------------------------------

/*******************************************************************\
| THRThreadCreate                                                   |
\*******************************************************************/

void THRThreadCreate (
  THRThread *pThread,
  void *(*pFunction) (void *),
  void *pArgument)
{
  int iDummy;

  iDummy = pthread_create (pThread, NULL, pFunction, pArgument);
  assert (!iDummy);
};

/*******************************************************************\
| THRThreadFinish                                                   |
\*******************************************************************/

void THRThreadFinish (THRThread *pThread)
{
  int iDummy;

  iDummy = pthread_join (*pThread, NULL);
  assert (!iDummy);
};

/*******************************************************************\
| THRThreadSleep                                                    |
\*******************************************************************/

void THRThreadSleep (int iSeconds)
{
  sleep (iSeconds);
};

//-------------------------------------------------------------------

/*******************************************************************\
| THRLockCreate                                                     |
\*******************************************************************/

void THRLockCreate (
  THRLock *pLock)
{
  int iDummy;

  // This here relies on the fast mutex being the default type
  // created, which is true on Linux and needed for benchmarks.

  iDummy = pthread_mutex_init (pLock,NULL);
  assert (!iDummy);
};

/*******************************************************************\
| THRLockFinish                                                     |
\*******************************************************************/

void THRLockFinish (THRLock *pLock)
{
  int iDummy;

  iDummy = pthread_mutex_destroy (pLock);
  assert (!iDummy);
};

/*******************************************************************\
| THRLockLock                                                       |
\*******************************************************************/

void THRLockLock (THRLock *pLock)
{
  int iDummy;

  iDummy = pthread_mutex_lock (pLock);
  assert (!iDummy);
};

/*******************************************************************\
| THRLockUnlock                                                     |
\*******************************************************************/

void THRLockUnlock (THRLock *pLock)
{
  int iDummy;

  iDummy = pthread_mutex_unlock (pLock);
  assert (!iDummy);
};

//-------------------------------------------------------------------
