/*******************************************************************\
| THR_Solaris_Threads.CC                                            |
+-------------------------------------------------------------------+
| The thread wrappers for Solaris threads.                          |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
\*******************************************************************/

#include <unistd.h>

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

//-------------------------------------------------------------------

/*******************************************************************\
| THRThreadCreate                                                   |
\*******************************************************************/

void THRThreadCreate (
  THRThread *pThread,
  void *(*pFunction) (void *),
  void *pArgument)
{
  int iDummy;

  iDummy = thr_create (NULL, 0, pFunction, pArgument, THR_BOUND, pThread);
  assert (!iDummy);
};

/*******************************************************************\
| THRThreadFinish                                                   |
\*******************************************************************/

void THRThreadFinish (THRThread *pThread)
{
  int iDummy;

  iDummy = thr_join (*pThread, NULL, NULL);
  assert (!iDummy);
};

/*******************************************************************\
| THRThreadSleep                                                    |
\*******************************************************************/

void THRThreadSleep (int iSeconds)
{
  sleep (iSeconds);
};

//-------------------------------------------------------------------

/*******************************************************************\
| THRLockCreate                                                     |
\*******************************************************************/

void THRLockCreate (
  THRLock *pLock)
{
  int iDummy;

  // Semaphores are used instead of mutexes because we need to
  // be able to disregard owners when unlocking.

  iDummy = sema_init (pLock, 1, 0, NULL);
  assert (!iDummy);
};

/*******************************************************************\
| THRLockFinish                                                     |
\*******************************************************************/

void THRLockFinish (THRLock *pLock)
{
  int iDummy;

  iDummy = sema_destroy (pLock);
  assert (!iDummy);
};

/*******************************************************************\
| THRLockLock                                                       |
\*******************************************************************/

void THRLockLock (THRLock *pLock)
{
  int iDummy;

  iDummy = sema_wait (pLock);
  assert (!iDummy);
};

/*******************************************************************\
| THRLockUnlock                                                     |
\*******************************************************************/

void THRLockUnlock (THRLock *pLock)
{
  int iDummy;

  iDummy = sema_post (pLock);
  assert (!iDummy);
};

//-------------------------------------------------------------------
