/*******************************************************************\
| THR_Windows_Threads.CC                                            |
+-------------------------------------------------------------------+
| The thread wrappers for Windows threads.                          |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               Adam Buble <adam.buble@mff.cuni.cz>                 |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

//-------------------------------------------------------------------

/*******************************************************************\
| THRThreadCreate                                                   |
\*******************************************************************/

void THRThreadCreate (
  THRThread *pThread,
  void *(*pFunction) (void *),
  void *pArgument)
{
  (*pThread) = CreateThread (NULL, 0, (LPTHREAD_START_ROUTINE) pFunction, pArgument, 0, NULL);
  assert ((*pThread) != NULL);
};

/*******************************************************************\
| THRThreadFinish                                                   |
\*******************************************************************/

void THRThreadFinish (THRThread *pThread)
{
  DWORD iDummyDWord;
  BOOL  iDummyBool;

  iDummyDWord = WaitForSingleObject (*pThread, INFINITE);
  assert (iDummyDWord == WAIT_OBJECT_0);

  iDummyBool = CloseHandle (*pThread);
  assert (iDummyBool);
};

/*******************************************************************\
| THRThreadSleep                                                    |
\*******************************************************************/

void THRThreadSleep (int iSeconds)
{
  Sleep (iSeconds * 1000);
};

//-------------------------------------------------------------------

/*******************************************************************\
| THRLockCreate                                                     |
|-------------------------------------------------------------------|
| Should you wonder why locks are implemented through semaphores    |
| and not mutexes, it is because the concept of owners interferes   |
| with the benchmarks.                                              |
\*******************************************************************/

void THRLockCreate (
  THRLock *pLock)
{
  (*pLock) = CreateSemaphore (NULL, 1, 1, NULL);
  assert ((*pLock) != NULL);
};

/*******************************************************************\
| THRLockFinish                                                     |
\*******************************************************************/

void THRLockFinish (THRLock *pLock)
{
  BOOL iDummyBool;

  iDummyBool = CloseHandle (*pLock);
  assert (iDummyBool);
};

/*******************************************************************\
| THRLockLock                                                       |
\*******************************************************************/

void THRLockLock (THRLock *pLock)
{
  DWORD iDummyDWord;

  iDummyDWord = WaitForSingleObject (*pLock, INFINITE);
  assert (iDummyDWord == WAIT_OBJECT_0);
};

/*******************************************************************\
| THRLockUnlock                                                     |
\*******************************************************************/

void THRLockUnlock (THRLock *pLock)
{
  BOOL iDummyBool;

  iDummyBool = ReleaseSemaphore (*pLock, 1, NULL);
  assert (iDummyBool);
};

//-------------------------------------------------------------------
