/*******************************************************************\
| TIM_Solaris_HRTime.CC                                             |
+-------------------------------------------------------------------+
| The timer functions for Solaris platforms with gethrtime.         |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               Adam Buble <adam.buble@mff.cuni.cz>                 |
\*******************************************************************/

#include <sys/time.h>

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

//-------------------------------------------------------------------

// TIM_CycleGranularity		How many samples to get granularity

#define TIM_CycleGranularity	100

//-------------------------------------------------------------------

uint64 iTIMTimerScale       = 0;
uint64 iTIMTimerGranularity = 0;

//-------------------------------------------------------------------

/*******************************************************************\
| TIMInit                                                           |
\*******************************************************************/

void TIMInit (void)
{
  // Determine the true granularity of the gethrtime call.

  uint64         iLastTime, iThisTime;
  uint64         iThisStep, iMinStep;
  int            iStepCount;

  iLastTime = TIMGetTimestamp ();
  iMinStep  = 0;

  for (iStepCount = 0 ; iStepCount < TIM_CycleGranularity ; iStepCount ++)
  {
    do iThisTime = TIMGetTimestamp ();
    while (iThisTime == iLastTime);
    iThisStep = iThisTime - iLastTime;
    if ((iStepCount == 0) || (iThisStep < iMinStep))
      iMinStep = iThisStep;
  };

  iTIMTimerScale       = 1000000000uLL;
  iTIMTimerGranularity = iMinStep;
};

/*******************************************************************\
| TIMGetTimestamp                                                   |
\*******************************************************************/

uint64 TIMGetTimestamp (void)
{
  return (gethrtime ());
};

//-------------------------------------------------------------------
