/*******************************************************************\
| Client.CC (Concurrency Client Connections)                        |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

#include "defs.h"

//-------------------------------------------------------------------

#define BEN_MaxLineLen          1024

#define BEN_Threads             10

// Do not use CORBA var classes as global variables, it causes
// problems with proper initialization and shutdown order.

Ping_ptr        pPing;
CORBA::ULong    iClient;
CORBA::ULong    iTotal;
CORBA::ULong    iOne;

//-------------------------------------------------------------------

/*******************************************************************\
| BENGetName                                                        |
\*******************************************************************/

const char *BENGetName (void)
{
  return ("Concurrency Client Connections Client");
}

//-------------------------------------------------------------------

/*******************************************************************\
| ThreadAction                                                      |
\*******************************************************************/

void *ThreadAction (void *pArgs)
{
  CORBA::ULong iOne;

  ORBEnvVar
  ORBExcTry
  {
    do
    {
      pPing->Pong (iOne ORBArgNext); ORBExcCheck
    }
    while (iOne != iClient);
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Client " << ORBExcShow)
  }

  return (NULL);
}

//-------------------------------------------------------------------

/*******************************************************************\
| ActionPing                                                        |
\*******************************************************************/

void ActionPing (tMEAArgs *pArgs ORBDclNext) ORBDclThrow
{
  pPing->Pong (iOne ORBArgNext);
}

//-------------------------------------------------------------------

/*******************************************************************\
| BENClient                                                         |
\*******************************************************************/

void BENClient (
  CORBA::Object_ptr pReference,
  int iArgC, char *apArgV [])
{
  THRThread             asThreads [BEN_Threads];
  uint32                iThread;

  char                  acLine [BEN_MaxLineLen];

  // Parse the command line

  switch (iArgC)
  {
    case 1:
      break;
    default:
      DSPFatal ("BEN", "Usage: " << apArgV [0]);
  }

  // Execute the measurement

  ORBEnvVar
  ORBExcTry
  {
    pPing = Ping::_narrow (pReference);

    pPing->Register (iClient, iTotal ORBArgNext); ORBExcCheck

    // Create threads to provide the load on clients that
    // will not be selected for the measurement itself.

    for (iThread = 0 ; iThread < BEN_Threads ; iThread ++)
      THRThreadCreate (&asThreads [iThread], ThreadAction, NULL);

    // The next phase of the benchmark starts when the
    // threads finish. The threads will finish only when
    // this client was selected for the measurement.

    for (iThread = 0 ; iThread < BEN_Threads ; iThread ++)
      THRThreadFinish (&asThreads [iThread]);

    snprintf (acLine, sizeof (acLine),
              "[1]\tClients\t%lu\t"
              "[1]\tThreads\t%lu\t",
              iTotal,
              BEN_Threads);
    MEAMeasure (acLine,
                ActionPing,
                NULL, 0,
                BEN_Threads,
                SAM_TestSamples
                ORBArgNext); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Client " << ORBExcShow)
  }
}

//-------------------------------------------------------------------
