/*******************************************************************\
| Server.CC (Concurrency Client Connections)                        |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

#include "defs.h"

//-------------------------------------------------------------------

CORBA::ULong    iClient = 0;
CORBA::ULong    iTotal  = 0;
CORBA::ULong    iOne    = 0;

THRLock         sLock;

//-------------------------------------------------------------------

/*******************************************************************\
| PingServant                                                       |
\*******************************************************************/

class PingServant : public POA_Ping
{
  public:

    void Pong (CORBA::ULong_out iOne ORBDclNext) ORBDclThrow;
    void Register (CORBA::ULong_out iClient, CORBA::ULong_out iTotal ORBDclNext) ORBDclThrow;
};

//-------------------------------------------------------------------

/*******************************************************************\
| PingServant::Pong                                                |
\*******************************************************************/

void PingServant::Pong (
  CORBA::ULong_out iClientOne
  ORBDclNext) ORBDclThrow
{
  // The iOne variable will change one time. Locking would
  // severely influence the tests though, which is why we
  // just hope that the assignment will be atomic.

  iClientOne = iOne;
}

/*******************************************************************\
| PingServant::Register                                            |
\*******************************************************************/

void PingServant::Register (
  CORBA::ULong_out iClientClient,
  CORBA::ULong_out iClientTotal
  ORBDclNext) ORBDclThrow
{
  THRLockLock (&sLock);

  iClient ++;

  assert (iClient <= iTotal);

  iClientClient = iClient;
  iClientTotal = iTotal;

  // The iOne variable will change one time. Locking would
  // severely influence the tests though, which is why we
  // just hope that the assignment will be atomic.

  if (iClient == iTotal)
  {
    iOne = RNDGetRandom (iTotal) + 1;
  }

  THRLockUnlock (&sLock);
}

//-------------------------------------------------------------------

/*******************************************************************\
| BENGetName                                                        |
\*******************************************************************/

const char *BENGetName (void)
{
  return ("Concurrency Client Connections Server");
};

/*******************************************************************\
| BENServer                                                         |
\*******************************************************************/

CORBA::Object_ptr BENServer (
  int iArgC, char *apArgV [])
{
  PingServant           *pPingServant;
  CORBA::Object_ptr     pPingObject;

  // Parse the command line

  switch (iArgC)
  {
        case 2:
          iTotal = atol (apArgV [1]);
          break;
        default:
          DSPFatal ("BEN", "Usage: " << apArgV [0] << " Clients");
  }

  // Execute the measurement

  ORBEnvVar
  ORBExcTry
  {
        // We do not call THRLockFinish on this particular lock
        // because there is no good place to do that, as we do
        // not get control when the server terminates. It
        // should not matter ...

        THRLockCreate (&sLock);

        pPingServant = new PingServant ();
        pPingObject = pPingServant->_this (ORBArgOnly); ORBExcCheck
  }
  ORBExcCatch
  {
        DSPFatal ("BEN", "Server " << ORBExcShow)
  };

  return (pPingObject);
};

//-------------------------------------------------------------------
