/*******************************************************************\
| Server.CC (Dispatch Servant Instances)                            |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

#include "defs.h"

//-------------------------------------------------------------------

/*******************************************************************\
| PingServant                                                       |
\*******************************************************************/

class PingServant : public POA_Ping, public PortableServer::RefCountServantBase
{
  public:

    void Pong (ORBDclOnly) ORBDclThrow;
};

//-------------------------------------------------------------------

/*******************************************************************\
| PingServant::Pong                                                |
\*******************************************************************/

void PingServant::Pong (
  ORBDclOnly) ORBDclThrow
{
};

//-------------------------------------------------------------------

/*******************************************************************\
| PingFactoryServant                                                |
\*******************************************************************/

class PingFactoryServant : public POA_PingFactory
{
  public:

    tPingSequence *Create (CORBA::ULong iLength ORBDclNext) ORBDclThrow;
};

//-------------------------------------------------------------------

/*******************************************************************\
| PingFactoryServant::Create                                        |
\*******************************************************************/

tPingSequence *PingFactoryServant::Create (
  CORBA::ULong iLength
  ORBDclNext) ORBDclThrow
{
  tPingSequence *pPingSequence;
  PingServant   *pPingServant;
  CORBA::ULong  iCount;

  ORBExcTry
  {
    pPingSequence = new tPingSequence (iLength);
    pPingSequence->length (iLength);

    for (iCount = 0 ; iCount < iLength ; iCount ++)
    {
      pPingServant = new PingServant ();
      (*pPingSequence) [iCount] = pPingServant->_this (ORBArgOnly); ORBExcCheck
    };
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Server " << ORBExcShow)
  };

  return (pPingSequence);
};

//-------------------------------------------------------------------

/*******************************************************************\
| BENGetName                                                        |
\*******************************************************************/

const char *BENGetName (void)
{
  return ("Dispatch Servant Instances Server");
};

/*******************************************************************\
| BENServer                                                         |
\*******************************************************************/

CORBA::Object_ptr BENServer (
  int iArgC, char *apArgV [])
{
  PingFactoryServant    *pPingFactoryServant;
  CORBA::Object_ptr     pPingFactoryObject;

  // Parse the command line

  switch (iArgC)
  {
    case 1:
      break;
    default:
      DSPFatal ("BEN", "Usage: " << apArgV [0]);
  };

  // Execute the measurement

  ORBEnvVar
  ORBExcTry
  {
    pPingFactoryServant = new PingFactoryServant ();
    pPingFactoryObject = pPingFactoryServant->_this (ORBArgOnly); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Server " << ORBExcShow)
  };

  return (pPingFactoryObject);
};

//-------------------------------------------------------------------
