/*******************************************************************\
| Client.CC (Dispatch Servant Lifecycle)                            |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

#include "defs.h"

//-------------------------------------------------------------------

#define BEN_MaxLineLen          1024

#define BEN_MinInstances        1
#define BEN_MaxInstances        SAM_MaxInstances
#define BEN_StepLimit           5
#define BEN_StepScale           10

// Do not use CORBA var classes as global variables, it causes
// problems with proper initialization and shutdown order.

EmptyFactory_ptr pEmptyFactory;

tMEAStamp asStamps [6] =
{
  { "CreateInvoke", MEA_StampRemote | MEA_StampAbsolute },
  { "CreateReturn", MEA_StampRemote | MEA_StampAbsolute },
  { "CreateFinish", MEA_StampLocal | MEA_StampAbsolute },
  { "DeleteInvoke", MEA_StampRemote },
  { "DeleteReturn", MEA_StampRemote },
  { "DeleteFinish", MEA_StampLocal }
};

#define BEN_StampCreateInvoke           0
#define BEN_StampCreateReturn           1
#define BEN_StampCreateFinish           2
#define BEN_StampDeleteInvoke           3
#define BEN_StampDeleteReturn           4
#define BEN_StampDeleteFinish           5

//-------------------------------------------------------------------

/*******************************************************************\
| BENGetName                                                        |
\*******************************************************************/

const char *BENGetName (void)
{
  return ("Dispatch Servant Lifecycle Client");
}

//-------------------------------------------------------------------

/*******************************************************************\
| ActionCreateDelete                                                |
\*******************************************************************/

void ActionCreateDelete (tMEAArgs *pArgs ORBDclNext) ORBDclThrow
{
  Empty_ptr pEmpty;
  pEmptyFactory->CreateIndividual (pEmpty,
                                   pArgs->aiTimes [BEN_StampCreateInvoke],
                                   pArgs->aiTimes [BEN_StampCreateReturn]
                                   ORBArgNext); ORBExcCheck
  pArgs->aiTimes [BEN_StampCreateFinish] = TIMGetTimestamp ();
  pEmptyFactory->DeleteIndividual (pEmpty,
                                   pArgs->aiTimes [BEN_StampDeleteInvoke],
                                   pArgs->aiTimes [BEN_StampDeleteReturn]
                                   ORBArgNext); ORBExcCheck
  CORBA::release (pEmpty ORBArgNext); ORBExcCheck
  pArgs->aiTimes [BEN_StampDeleteFinish] = TIMGetTimestamp ();

  pArgs->aiTimes [BEN_StampDeleteInvoke] -= pArgs->aiTimes [BEN_StampCreateFinish];
  pArgs->aiTimes [BEN_StampDeleteReturn] -= pArgs->aiTimes [BEN_StampCreateFinish];
  pArgs->aiTimes [BEN_StampDeleteFinish] -= pArgs->aiTimes [BEN_StampCreateFinish];
}

//-------------------------------------------------------------------

/*******************************************************************\
| BENClient                                                         |
\*******************************************************************/

void BENClient (
  CORBA::Object_ptr pReference,
  int iArgC, char *apArgV [])
{
  CORBA::ULong          iEmptySequenceLength;

  uint32                iInstancesMin = BEN_MinInstances, iInstancesMax = BEN_MaxInstances;
  uint32                iInstancesLimit = BEN_StepLimit, iInstancesScale = BEN_StepScale;
  uint32                iInstancesStep, iInstancesCount;
  uint32                iInstance;

  char                  acLine [BEN_MaxLineLen];

  // Parse the command line

  switch (iArgC)
  {
    case 5:
      iInstancesMin     = atol (apArgV [1]);
      iInstancesMax     = atol (apArgV [2]);
      iInstancesLimit   = atol (apArgV [3]);
      iInstancesScale   = atol (apArgV [4]);
    case 1:
      break;
    default:
      DSPFatal ("BEN", "Usage: " << apArgV [0] << " [InstancesMin Max Limit Scale]");
  }

  // Execute the measurement

  ORBEnvVar
  ORBExcTry
  {
    pEmptyFactory = EmptyFactory::_narrow (pReference);

    iEmptySequenceLength = 0;

    MEAInitStep (iInstancesStep, iInstancesCount, iInstancesMin, iInstancesScale);
    while (iInstancesCount <= iInstancesMax)
    {
      // The additional objects for this step of the benchmark are
      // created here. The reference to the sequence holding them
      // is not kept because the sequence is not needed for
      // anything except to exist in memory. Hence, this is
      // not a memory leak but a valid part of the benchmark.

      pEmptyFactory->CreateSequence (iInstancesCount - iEmptySequenceLength ORBArgNext); ORBExcCheck
      iEmptySequenceLength = iInstancesCount;

      // Execute the measurement for all action stages.

      snprintf (acLine, sizeof (acLine),
                "[1]\tInstances\t%lu\t",
                iInstancesCount);
      MEAMeasure (acLine,
                  ActionCreateDelete,
                  asStamps, sizeofarray (asStamps),
                  1,
                  SAM_TestSamples
                  ORBArgNext); ORBExcCheck

      // There is no deallocation of servants.

      MEANextStep (iInstancesStep, iInstancesCount, iInstancesLimit, iInstancesScale);
    }
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Client " << ORBExcShow)
  }
}

//-------------------------------------------------------------------

