/*******************************************************************\
| Server.CC (Dispatch Servant Lifecycle)                            |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

#include "defs.h"

//-------------------------------------------------------------------

THRLock sDeleteLock;

//-------------------------------------------------------------------

/*******************************************************************\
| EmptyServant                                                      |
\*******************************************************************/

class EmptyServant : public POA_Empty, public PortableServer::RefCountServantBase
{
  public:

    ~EmptyServant (void);
};

//-------------------------------------------------------------------

/*******************************************************************\
| EmptyServant::~EmptyServant                                       |
\*******************************************************************/

EmptyServant::~EmptyServant (void)
{
  THRLockUnlock (&sDeleteLock);
}

//-------------------------------------------------------------------

/*******************************************************************\
| EmptyFactoryServant                                               |
\*******************************************************************/

class EmptyFactoryServant : public POA_EmptyFactory
{
  public:

    void CreateIndividual (Empty_out pEmpty, CORBA::ULongLong_out iTimeInvoke, CORBA::ULongLong_out iTimeReturn ORBDclNext) ORBDclThrow;
    void DeleteIndividual (Empty_ptr pEmpty, CORBA::ULongLong_out iTimeInvoke, CORBA::ULongLong_out iTimeReturn ORBDclNext) ORBDclThrow;
    tEmptySequence *CreateSequence (CORBA::ULong iLength ORBDclNext) ORBDclThrow;
};

//-------------------------------------------------------------------

/*******************************************************************\
| EmptyFactoryServant::CreateIndividual                             |
\*******************************************************************/

void EmptyFactoryServant::CreateIndividual (
  Empty_out pEmpty,
  CORBA::ULongLong_out iTimeInvoke,
  CORBA::ULongLong_out iTimeReturn
  ORBDclNext) ORBDclThrow
{
  iTimeInvoke = TIMGetTimestamp ();

  ORBExcTry
  {
    EmptyServant *pEmptyServant = new EmptyServant ();
    pEmpty = pEmptyServant->_this (ORBArgOnly); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Server " << ORBExcShow)
  };

  iTimeReturn = TIMGetTimestamp ();
};

/*******************************************************************\
| EmptyFactoryServant::DeleteIndividual                             |
\*******************************************************************/

void EmptyFactoryServant::DeleteIndividual (
  Empty_ptr pEmpty,
  CORBA::ULongLong_out iTimeInvoke,
  CORBA::ULongLong_out iTimeReturn
  ORBDclNext) ORBDclThrow
{
  iTimeInvoke = TIMGetTimestamp ();

  ORBExcTry
  {
    PortableServer::ObjectId_var vEmptyOID = pRootPOA->reference_to_id (pEmpty ORBArgNext); ORBExcCheck
    PortableServer::ServantBase *pEmptyServant = pRootPOA->reference_to_servant (pEmpty ORBArgNext); ORBExcCheck
    pRootPOA->deactivate_object (vEmptyOID ORBArgNext); ORBExcCheck
    pEmptyServant->_remove_ref ();
    pEmptyServant->_remove_ref ();
    THRLockLock (&sDeleteLock);
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Server " << ORBExcShow)
  };

  iTimeReturn = TIMGetTimestamp ();
};

/*******************************************************************\
| EmptyFactoryServant::CreateSequence                               |
\*******************************************************************/

tEmptySequence *EmptyFactoryServant::CreateSequence (
  CORBA::ULong iLength
  ORBDclNext) ORBDclThrow
{
  tEmptySequence *pEmptySequence;
  EmptyServant   *pEmptyServant;
  CORBA::ULong  iCount;

  ORBExcTry
  {
    pEmptySequence = new tEmptySequence (iLength);
    pEmptySequence->length (iLength);

    for (iCount = 0 ; iCount < iLength ; iCount ++)
    {
      pEmptyServant = new EmptyServant ();
      (*pEmptySequence) [iCount] = pEmptyServant->_this (ORBArgOnly); ORBExcCheck
    };
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Server " << ORBExcShow)
  };

  return (pEmptySequence);
};

//-------------------------------------------------------------------

/*******************************************************************\
| BENGetName                                                        |
\*******************************************************************/

const char *BENGetName (void)
{
  return ("Dispatch Servant Lifecycle Server");
};

/*******************************************************************\
| BENServer                                                         |
\*******************************************************************/

CORBA::Object_ptr BENServer (
  int iArgC, char *apArgV [])
{
  EmptyFactoryServant    *pEmptyFactoryServant;
  CORBA::Object_ptr     pEmptyFactoryObject;

  // Parse the command line

  switch (iArgC)
  {
    case 1:
      break;
    default:
      DSPFatal ("BEN", "Usage: " << apArgV [0]);
  };

  // Execute the measurement

  ORBEnvVar
  ORBExcTry
  {
    THRLockCreate (&sDeleteLock);
    THRLockLock (&sDeleteLock);

    pEmptyFactoryServant = new EmptyFactoryServant ();
    pEmptyFactoryObject = pEmptyFactoryServant->_this (ORBArgOnly); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Server " << ORBExcShow)
  };

  return (pEmptyFactoryObject);
};

//-------------------------------------------------------------------
