/*******************************************************************\
| Server.CC (Invocation Dynamic)                                    |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

#include "defs.h"

//-------------------------------------------------------------------

/*******************************************************************\
| PingServant                                                       |
\*******************************************************************/

class PingServant : public PortableServer::DynamicImplementation
{
  public:

    void invoke (CORBA::ServerRequest_ptr pRequest ORBDclNext) ORBDclThrow;
    CORBA::RepositoryId _primary_interface (const PortableServer::ObjectId &oID, PortableServer::POA_ptr pPOA ORBDclNext) ORBDclThrow;
};

//-------------------------------------------------------------------

/*******************************************************************\
| PingServant::invoke                                              |
\*******************************************************************/

void PingServant::invoke (
  CORBA::ServerRequest_ptr pRequest
  ORBDclNext) ORBDclThrow
{
  if (!strcmp (pRequest->operation (), "Pong"))
  {
    // Note that calling arguments is mandated by the
    // standard even when the function has none.

    CORBA::NVList_ptr pArguments;
    CORBA::Any        oResult;

    pORB->create_list (0, pArguments ORBArgNext);
    pRequest->arguments (pArguments ORBArgNext);
    oResult <<= (CORBA::ULongLong) TIMGetTimestamp ();
    pRequest->set_result (oResult ORBArgNext);
  }
  else
  {
    assert (FALSE);
  };
};

/*******************************************************************\
| PingServant::_primary_interface                                  |
\*******************************************************************/

CORBA::RepositoryId PingServant::_primary_interface (
  const PortableServer::ObjectId &oID,
  PortableServer::POA_ptr pPOA
  ORBDclNext) ORBDclThrow
{
  return (CORBA::string_dup ("IDL:Ping:1.0"));
};

//-------------------------------------------------------------------

/*******************************************************************\
| BENGetName                                                        |
\*******************************************************************/

const char *BENGetName (void)
{
  return ("Invocation Dynamic Server");
};

/*******************************************************************\
| BENServer                                                         |
\*******************************************************************/

CORBA::Object_ptr BENServer (
  int iArgC, char *apArgV [])
{
  PingServant                   *pPingServant;
  CORBA::Object_ptr             pPingObject;

  // Parse the command line

  switch (iArgC)
  {
    case 1:
      break;
    default:
      DSPFatal ("BEN", "Usage: " << apArgV [0]);
  };

  // Execute the measurement

  ORBEnvVar
  ORBExcTry
  {
    pPingServant = new PingServant ();
    pPingObject = pRootPOA->servant_to_reference (pPingServant ORBArgNext); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Server " << ORBExcShow)
  };

  return (pPingObject);
};

//-------------------------------------------------------------------
