/*******************************************************************\
| Client.CC (Invocation Initial)                                    |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

#include "defs.h"

//-------------------------------------------------------------------

// Do not use CORBA var classes as global variables, it causes
// problems with proper initialization and shutdown order.

PingFactory_ptr pPingFactory;

tMEAStamp asStamps [3] =
{
  { "RoundtripFirst",  MEA_StampLocal },
  { "RoundtripSecond", MEA_StampLocal },
  { "RoundtripThird",  MEA_StampLocal }
};

#define BEN_StampRoundtrip1st           0
#define BEN_StampRoundtrip2nd           1
#define BEN_StampRoundtrip3rd           2

//-------------------------------------------------------------------

/*******************************************************************\
| BENGetName                                                        |
\*******************************************************************/

const char *BENGetName (void)
{
  return ("Invocation Initial Client");
}

//-------------------------------------------------------------------

/*******************************************************************\
| ActionFork                                                        |
+-------------------------------------------------------------------+
| Three Pongs may be Three Prongs may be One Fork :-) ...           |
\*******************************************************************/

void ActionFork (tMEAArgs *pArgs ORBDclNext) ORBDclThrow
{
  uint64        iTimeBefore;
  Ping_ptr      pPing;

  // Create the Ping object

  pPingFactory->CreateIndividual (pPing ORBArgNext); ORBExcCheck

  // Measure three first Pongs

  iTimeBefore = TIMGetTimestamp ();
  pPing->Pong (ORBArgOnly); ORBExcCheck
  pArgs->aiTimes [BEN_StampRoundtrip1st] = TIMGetTimestamp ();
  pPing->Pong (ORBArgOnly); ORBExcCheck
  pArgs->aiTimes [BEN_StampRoundtrip2nd] = TIMGetTimestamp ();
  pPing->Pong (ORBArgOnly); ORBExcCheck
  pArgs->aiTimes [BEN_StampRoundtrip3rd] = TIMGetTimestamp ();

  pArgs->aiTimes [BEN_StampRoundtrip3rd] -= pArgs->aiTimes [BEN_StampRoundtrip2nd];
  pArgs->aiTimes [BEN_StampRoundtrip2nd] -= pArgs->aiTimes [BEN_StampRoundtrip1st];
  pArgs->aiTimes [BEN_StampRoundtrip1st] -= iTimeBefore;

  // Delete the object

  pPingFactory->DeleteIndividual (pPing ORBArgNext); ORBExcCheck
  CORBA::release (pPing ORBArgNext); ORBExcCheck
}

//-------------------------------------------------------------------

/*******************************************************************\
| BENClient                                                         |
\*******************************************************************/

void BENClient (
  CORBA::Object_ptr pReference,
  int iArgC, char *apArgV [])
{
  // Parse the command line

  switch (iArgC)
  {
    case 1:
      break;
    default:
      DSPFatal ("BEN", "Usage: " << apArgV [0]);
  }

  // Execute the measurement

  ORBEnvVar
  ORBExcTry
  {
    pPingFactory = PingFactory::_narrow (pReference);

    // Execute the measurement for all action stages.

    MEAMeasure ("",
                ActionFork,
                asStamps, sizeofarray (asStamps),
                1,
                SAM_TestSamples
                ORBArgNext); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Client " << ORBExcShow)
  }
}

//-------------------------------------------------------------------
