/*******************************************************************\
| Server.CC (Invocation Initial)                                    |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

#include "defs.h"

//-------------------------------------------------------------------

THRLock sDeleteLock;

//-------------------------------------------------------------------

/*******************************************************************\
| PingServant                                                       |
\*******************************************************************/

class PingServant : public POA_Ping, public PortableServer::RefCountServantBase
{
  public:

    ~PingServant (void);
    void Pong (ORBDclOnly) ORBDclThrow;
};

//-------------------------------------------------------------------

/*******************************************************************\
| PingServant::~PingServant                                         |
\*******************************************************************/

PingServant::~PingServant (void)
{
  THRLockUnlock (&sDeleteLock);
}

/*******************************************************************\
| PingServant::Pong                                                 |
\*******************************************************************/

void PingServant::Pong (
  ORBDclOnly) ORBDclThrow
{
}

//-------------------------------------------------------------------

/*******************************************************************\
| PingFactoryServant                                                |
\*******************************************************************/

class PingFactoryServant : public POA_PingFactory
{
  public:

    void CreateIndividual (Ping_out pPing ORBDclNext) ORBDclThrow;
    void DeleteIndividual (Ping_ptr pPing ORBDclNext) ORBDclThrow;
};

//-------------------------------------------------------------------

/*******************************************************************\
| PingFactoryServant::CreateIndividual                              |
\*******************************************************************/

void PingFactoryServant::CreateIndividual (
  Ping_out pPing
  ORBDclNext) ORBDclThrow
{
  ORBExcTry
  {
    PingServant *pPingServant = new PingServant ();
    pPing = pPingServant->_this (ORBArgOnly); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Server " << ORBExcShow)
  }
}

/*******************************************************************\
| PingFactoryServant::DeleteIndividual                              |
\*******************************************************************/

void PingFactoryServant::DeleteIndividual (
  Ping_ptr pPing
  ORBDclNext) ORBDclThrow
{
  ORBExcTry
  {
    PortableServer::ObjectId_var vPingOID = pRootPOA->reference_to_id (pPing ORBArgNext); ORBExcCheck
    PortableServer::ServantBase *pPingServant = pRootPOA->reference_to_servant (pPing ORBArgNext); ORBExcCheck
    pRootPOA->deactivate_object (vPingOID ORBArgNext); ORBExcCheck
    pPingServant->_remove_ref ();
    pPingServant->_remove_ref ();
    THRLockLock (&sDeleteLock);
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Server " << ORBExcShow)
  }
}

//-------------------------------------------------------------------

/*******************************************************************\
| BENGetName                                                        |
\*******************************************************************/

const char *BENGetName (void)
{
  return ("Invocation Initial Server");
}

/*******************************************************************\
| BENServer                                                         |
\*******************************************************************/

CORBA::Object_ptr BENServer (
  int iArgC, char *apArgV [])
{
  PingFactoryServant    *pPingFactoryServant;
  CORBA::Object_ptr     pPingFactoryObject;
  
  // Parse the command line

  switch (iArgC)
  {
    case 1:
      break;
    default:
      DSPFatal ("BEN", "Usage: " << apArgV [0]);
  }

  // Execute the measurement
  
  ORBEnvVar
  ORBExcTry
  {
    THRLockCreate (&sDeleteLock);
    THRLockLock (&sDeleteLock);
    
    pPingFactoryServant = new PingFactoryServant ();
    pPingFactoryObject = pPingFactoryServant->_this (ORBArgOnly); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Server " << ORBExcShow)
  }
  
  return (pPingFactoryObject);
}

//-------------------------------------------------------------------
