/*******************************************************************\
| Client.CC (Invocation Static)                                     |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

#include "defs.h"

//-------------------------------------------------------------------

#define BEN_MaxLineLen          1024

#define BEN_MinThreads          1
#define BEN_MaxThreads          SAM_MaxThreads
#define BEN_StepLimit           5
#define BEN_StepScale           10

// Do not use CORBA var classes as global variables, it causes
// problems with proper initialization and shutdown order.

Ping_ptr        pPing;

tMEAStamp asStamps [1] =
{
  { "Halfway", MEA_StampRemote | MEA_StampAbsolute },
};

//-------------------------------------------------------------------

/*******************************************************************\
| BENGetName                                                        |
\*******************************************************************/

const char *BENGetName (void)
{
  return ("Invocation Static Client");
};

//-------------------------------------------------------------------

/*******************************************************************\
| ActionPing                                                        |
\*******************************************************************/

void ActionPing (tMEAArgs *pArgs ORBDclNext) ORBDclThrow
{
  pArgs->aiTimes [0] = pPing->Pong (ORBArgOnly);
};

//-------------------------------------------------------------------

/*******************************************************************\
| BENClient                                                         |
\*******************************************************************/

void BENClient (
  CORBA::Object_ptr pReference,
  int iArgC, char *apArgV [])
{
  uint32        iThreadsMin = BEN_MinThreads, iThreadsMax = BEN_MaxThreads;
  uint32        iThreadsLimit = BEN_StepLimit, iThreadsScale = BEN_StepScale;
  uint32        iThreadsStep, iThreadsCount;
  char          acLine [BEN_MaxLineLen];

  // Parse the command line

  switch (iArgC)
  {
    case 5:
      iThreadsMin   = atol (apArgV [1]);
      iThreadsMax   = atol (apArgV [2]);
      iThreadsLimit = atol (apArgV [3]);
      iThreadsScale = atol (apArgV [4]);
    case 1:
      break;
    default:
      DSPFatal ("BEN", "Usage: " << apArgV [0] << " [ThreadsMin Max Limit Scale]");
  };

  // Execute the measurement

  ORBEnvVar
  ORBExcTry
  {
    pPing = Ping::_narrow (pReference);

    MEAInitStep (iThreadsStep, iThreadsCount, iThreadsMin, iThreadsScale);
    while (iThreadsCount <= iThreadsMax)
    {
      snprintf (acLine, sizeof (acLine),
                "[1]\tThreads\t%lu\t",
                iThreadsCount);
      MEAMeasure (acLine,
                  ActionPing,
                  asStamps, sizeofarray (asStamps),
                  iThreadsCount,
                  SAM_TestSamples
                  ORBArgNext); ORBExcCheck

      MEANextStep (iThreadsStep, iThreadsCount, iThreadsLimit, iThreadsScale);
    };
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Client " << ORBExcShow)
  };
};

//-------------------------------------------------------------------

