/*******************************************************************\
| Server.CC (Marshal Delayed Any)                                   |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

#include "defs.h"

//-------------------------------------------------------------------

// Do not use CORBA var classes as global variables, it causes
// problems with proper initialization and shutdown order.

tOctetArray_slice       *pOctetArray;
tOctetArray_forany      *pOctetArrayForAny;
CORBA::Any              *pOctetArrayAny;

//-------------------------------------------------------------------

/*******************************************************************\
| PingServant                                                       |
\*******************************************************************/

class PingServant : public POA_Ping
{
  public:
    void Pong (const CORBA::Any &oAny ORBDclNext) ORBDclThrow;
};

//-------------------------------------------------------------------

/*******************************************************************\
| PingServant::Pong                                                |
\*******************************************************************/

void PingServant::Pong (const CORBA::Any &oAny ORBDclNext) ORBDclThrow
{
  #if   defined BENAccessAccessed
    oAny >>= (*pOctetArrayForAny);
  #elif defined BENAccessIgnored
    (*pOctetArrayAny) >>= (*pOctetArrayForAny);
  #else
    #error
  #endif
}

//-------------------------------------------------------------------

/*******************************************************************\
| BENGetName                                                        |
\*******************************************************************/

const char *BENGetName (void)
{
  return ("Marshal Delayed Any Server");
};

/*******************************************************************\
| BENServer                                                         |
\*******************************************************************/

CORBA::Object_ptr BENServer (
  int iArgC, char *apArgV [])
{
  PingServant           *pPingServant;
  CORBA::Object_ptr     pPingObject;
  
  // Parse the command line

  switch (iArgC)
  {
    case 1:
      break;
    default:
      DSPFatal ("BEN", "Usage: " << apArgV [0]);
  };

  // Execute the measurement
  
  ORBEnvVar
  ORBExcTry
  {
    pOctetArray = tOctetArray_alloc ();
    pOctetArrayForAny = new tOctetArray_forany (pOctetArray);
    pOctetArrayAny = new CORBA::Any;
    (*pOctetArrayAny) <<= (*pOctetArrayForAny);

    pPingServant = new PingServant ();
    pPingObject = pPingServant->_this (ORBArgOnly); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Server " << ORBExcShow)
  };
  
  return (pPingObject);
};

//-------------------------------------------------------------------
