/*******************************************************************\
| Server.CC (Marshal Octet Sequence)                                |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include "com_defs.h"
#include "orb_defs.h"
#include "thr_defs.h"

#include "shr_defs.h"
#include "all_defs.h"

#include "defs.h"

//-------------------------------------------------------------------

/*******************************************************************\
| PingServant                                                       |
\*******************************************************************/

class PingServant : public POA_Ping
{
  public:

  #if   defined BENDirectionIN
    void Pong (const tOctetSequence &sSequence ORBDclNext) ORBDclThrow;
  #elif defined BENDirectionOUT
    void Pong (CORBA::ULong iLength, tOctetSequence_out pSequence ORBDclNext) ORBDclThrow;
  #elif defined BENDirectionINOUT
    void Pong (tOctetSequence &sSequence ORBDclNext) ORBDclThrow;
  #elif defined BENDirectionRETURN
    tOctetSequence *Pong (CORBA::ULong iLength ORBDclNext) ORBDclThrow;
  #else
    #error
  #endif
};

//-------------------------------------------------------------------

/*******************************************************************\
| PingServant::Pong                                                |
\*******************************************************************/

#if   defined BENDirectionIN
  void PingServant::Pong (const tOctetSequence &sSequence ORBDclNext) ORBDclThrow { };
#elif defined BENDirectionOUT
  void PingServant::Pong (CORBA::ULong iLength, tOctetSequence_out pSequence ORBDclNext) ORBDclThrow
  {
    pSequence = new tOctetSequence (iLength);
    pSequence->length (iLength);
  };
#elif defined BENDirectionINOUT
  void PingServant::Pong (tOctetSequence &sSequence ORBDclNext) ORBDclThrow { };
#elif defined BENDirectionRETURN
  tOctetSequence *PingServant::Pong (CORBA::ULong iLength ORBDclNext) ORBDclThrow
  {
    tOctetSequence *pSequence;
    pSequence = new tOctetSequence (iLength);
    pSequence->length (iLength);
    return (pSequence);
  };
#else
  #error
#endif

//-------------------------------------------------------------------

/*******************************************************************\
| BENGetName                                                        |
\*******************************************************************/

const char *BENGetName (void)
{
  return ("Marshal Octet Sequence Server");
};

/*******************************************************************\
| BENServer                                                         |
\*******************************************************************/

CORBA::Object_ptr BENServer (
  int iArgC, char *apArgV [])
{
  PingServant           *pPingServant;
  CORBA::Object_ptr     pPingObject;

  // Parse the command line

  switch (iArgC)
  {
    case 1:
      break;
    default:
      DSPFatal ("BEN", "Usage: " << apArgV [0]);
  };

  // Execute the measurement

  ORBEnvVar
  ORBExcTry
  {
    pPingServant = new PingServant ();
    pPingObject = pPingServant->_this (ORBArgOnly); ORBExcCheck
  }
  ORBExcCatch
  {
    DSPFatal ("BEN", "Server " << ORBExcShow)
  };

  return (pPingObject);
};

//-------------------------------------------------------------------
