/*******************************************************************\
| BenchBroker.java                                                  |
+-------------------------------------------------------------------+
| Copyright (C) Lubomir Bulej <lubomir.bulej@mff.cuni.cz>           |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

import java.util.Map;
import java.util.Hashtable;

import org.omg.CORBA.ORB;
import org.omg.CORBA.SystemException;

import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;


/*******************************************************************\
| BenchBroker                                                       |
\*******************************************************************/

public final class BenchBroker
{
	public static ORB 			theORB;
	public static POA 			rootPOA;
	public static POAManager 	rootPOAManager;
	public static String		orbName;

	/**
	 * Map for translation of ORB class names to ORB names.
	 * It is used to ensure that we run the benchmark only
	 * with supported ORBs.
	 */
	private static Map			orbClassMap;

	static {
		orbClassMap = new Hashtable ();
		orbClassMap.put ("org.jacorb.orb.ORB", "JacORB");
		orbClassMap.put ("org.openorb.CORBA.ORB", "OpenORB");
		orbClassMap.put ("com.inprise.vbroker.orb.ORB", "VisiBroker");
		orbClassMap.put ("com.iona.corba.art.artimpl.ORBImpl", "Orbix");
		orbClassMap.put ("org.objectweb.david.libs.binding.orbs.iiop.IIOPORB", "Jonathan");
		orbClassMap.put ("edu.uci.ece.zen.orb.ORB", "ZEN");
	}


	/**
	 * Initializes the ORB in benchmark client.
	 */
	public static void initClient (String [] args) {
		try {
			theORB = ORB.init (args, null);

			/*
			 * Make sure we use one of the allowed brokers.
			 */
			Class orbClass = theORB.getClass ();
			String clsName = orbClass.getName ();

			orbName = (String) orbClassMap.get (clsName);
			if (orbName == null) {
				throw new Exception ("Unsupported ORB implementation! ("+ clsName +")");
			}


		} catch (SystemException ex) {
			BenchGlobal.displayFatal ("ORB", "theORB "+ ex.toString ());
		} catch (Exception ex) {
			BenchGlobal.displayFatal ("ORB", "initClient "+ ex.getMessage ());
		}
	} // initClient (String [])


	/**
	 * Initializes the ORB in benchmark server.
	 */
	public static void initServer (String [] args) {
		initClient (args);

		try {
			org.omg.CORBA.Object basePOA = theORB.resolve_initial_references ("RootPOA");
			rootPOA = POAHelper.narrow (basePOA);
			rootPOAManager = rootPOA.the_POAManager ();

		} catch (SystemException ex) {
			BenchGlobal.displayFatal ("ORB", "RootPOA "+ ex.toString ());
		} catch (Exception ex) {
			BenchGlobal.displayFatal ("ORB", "initServer "+ ex.getMessage ());
		}
	} // initServer (String [])


	/**
	 * Shuts down (optionally) and destroys the ORB.
	 */
	public static void done (boolean shutdown) {
		if (theORB == null) {
			return;
		}

		try {
			if (shutdown) {
				theORB.shutdown (true);
			}

			/*
			 * Calling destroy() on the ORB should not be conditional, but
			 * OpenORB as of 1.3.1 still does not support the method.
			 */
			if (! orbName.equals ("OpenORB")) {
				theORB.destroy ();
			}
			theORB = null;

		} catch (SystemException ex) {
			BenchGlobal.displayFatal ("ORB", "done  "+ ex.toString ());
		}
	} // done ()

} // public final class BenchBroker

