/*******************************************************************\
| ServerMeter.java                                                  |
+-------------------------------------------------------------------+
| Copyright (C) Lubomir Bulej <lubomir.bulej@mff.cuni.cz>           |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

import org.omg.CORBA.LongHolder;
import org.omg.CORBA.UserException;
import org.omg.CORBA.SystemException;


/*******************************************************************\
| BenchServer                                                       |
\*******************************************************************/

public final class BenchServer {

	public static Server benchServer = null;


	public static void main (String args []) {

		BenchBroker.initServer (args);

		try {
			/*
			 * Display benchmark header and
			 * timer configuration.
			 */
			BenchGlobal.displayHeader (
				SuiteServer.getName ());

			BenchGlobal.displayConfig (
				SysInformer.timTimerScale,
				SysInformer.timTimerGranularity);

			// register benchmark & suite servers
			ServerServant benchServant = new ServerServant ();
			org.omg.CORBA.Object benchRef = BenchBroker.rootPOA.servant_to_reference (benchServant);
			benchServer = ServerHelper.narrow (benchRef);

			PeerLocator.announceServer (benchServer);
			benchServant.suiteServer = SuiteServer.create ();

			// process requests
			BenchBroker.rootPOAManager.activate ();
			BenchBroker.theORB.run ();

		} catch (SystemException ex) {
			BenchGlobal.displayFatal ("SRV", "Main "+ ex.toString ());
		} catch (UserException ex) {
			BenchGlobal.displayFatal ("SRV", "Main "+ ex.toString ());
		}

		BenchBroker.done (false);
	} // main (String [])

} // public final class BenchServer


/*******************************************************************\
| UsageMeter                                                        |
\*******************************************************************/

/**
 * Resource usage collector thread.
 *
 * We cannot afford to measure the resource usage during method
 * invocation, because that can block other method invocations.
 * The resource usage measurement is therefore done in a
 * separate thread, whose termination is queried by
 * polling, hopefully infrequently enough.
 */
final class UsageMeter implements Runnable {

	/**
	 * Indicates that collection of usage
	 * data is in progress.
	 */
	public boolean running;


	public UsageMeter () {
		running = false;
	} // ThreadUsage ()


	public final void run () {
		MeasureCore.usage ();
		running = false;
	} // run ()

}  // final class UsageMeter


/*******************************************************************\
| ServerServant                                                     |
\*******************************************************************/

/**
 * The benchmark server servant.
 */
final class ServerServant extends ServerPOA {

	private Thread 				usageThread;
	private UsageMeter 			usageMeter;

	public org.omg.CORBA.Object suiteServer;


	/**
	 * Benchmark server servant constructor. Instantiates
	 * usage meter object and associates it with a thread.
	 */
	public ServerServant () {
		usageMeter = new UsageMeter ();
	} // ServerServant ()


	/**
	 * Prints a string to standard output on server.
	 */
	public final void PrintString (String sMessage) {
		System.out.print (sMessage);
	} // PrintString (String)


	/**
	 * Starts the resource usage collector thread.
	 */
	public final void UsageStart () {
		if (! usageMeter.running) {
			usageThread = new Thread (usageMeter);
			usageMeter.running = true;
			usageThread.start ();
		}
	} // UsageStart ()


	/**
	 * Returns the status of the resource usage collector thread.
	 * This is used by the benchmark client to poll for completion
	 * of the colletor thread.
	 */
	public final boolean UsageQuery () {
		if (! usageMeter.running) {
			MeasureCore.joinThread (usageThread);
		}

		return usageMeter.running;
	} // UsageQuery ()


	/**
	 * Returns timer configuration and current system
	 * timer state on server.
	 */
	public final void TimerQuery (
		LongHolder iTimestamp,
		LongHolder iScale,
		LongHolder iGranularity) {

		iTimestamp.value	= SysInformer.timGetTimestamp ();
		iScale.value		= SysInformer.timTimerScale;
		iGranularity.value	= SysInformer.timTimerGranularity;
	} // TimerQuery (LongHolder, LongHolder, LongHolder)


	/**
	 * Returns a reference to suite server.
	 */
	public final org.omg.CORBA.Object Reference () {
		/*
		 * The reference does not need to be _duplicated
		 * because the memory is managed by JVM.
		 */
		return suiteServer;
	} // Reference ()


	/**
	 * Shuts down the server-side ORB. Since we're shutting it down
	 * from thread servicing the request, we must not wait for
	 * completion.
	 */
	public final void Terminate () {
		BenchBroker.theORB.shutdown (false);
	} // Terminate ()

} // final class ServerServant


