/*******************************************************************\
| LongSampler.java                                                  |
+-------------------------------------------------------------------+
| Copyright (C) Lubomir Bulej <lubomir.bulej@mff.cuni.cz>           |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/


/*******************************************************************\
| LongSampler                                                       |
\*******************************************************************/

/**
 * Holder for samples whose minimum, average
 * and maximum values are what is collected.
 */
public final class LongSampler {
	private long 		min = 0;
	private long 		avg = 0;
	private long 		max = 0;

	private boolean 	initialized = false;


	public final void initialize () {
		min = 0;
		avg = 0;
		max = 0;

		initialized = false;
	} // initialize ()


	public final void update (long val) {
		if (! initialized) {
			min = val;
			avg = val;
			max = val;
			initialized = true;

		} else {
			avg += val;

			if (val > max) {
				max = val;
			}

			if (val < min) {
				min = val;
			}
		}
	} // update (long)


	public final void update (LongSampler val) {
		if (! val.initialized) {
			return;
		}

		if (! initialized) {
			min = val.min;
			avg = val.avg;
			max = val.max;
			initialized = true;

		} else {
			avg += val.avg;

			if (val.min < min) {
				min = val.min;
			}

			if (val.max > max) {
				max = val.max;
			}
		}
	} // update (LongSampler)


	public final void normalize (long count) {
		if (count != 0) {
			avg /= count;
		}
	} // normalize (long)


	public final void print (String header) {
		System.out.print ("[3]\t"+ header +"\t"+
			min +"\t"+ avg +"\t"+ max +"\t");
	} // print (String)

} // public final class LongSampler
