/*******************************************************************\
| SysInformer.cc                                                    |
+-------------------------------------------------------------------+
| JNI v1.2 wrappers to various system internals.                    |
+-------------------------------------------------------------------+
| Copyright (C) Lubomir Bulej <lubomir.bulej@mff.cuni.cz>           |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

#include "com_defs.h"
#include "thr_defs.h"
#include "shr_defs.h"

#include "SysInformer.h"

//-------------------------------------------------------------------

/*******************************************************************\
| Java Native Interface v1.2                                        |
\*******************************************************************/

JNIEXPORT jint JNICALL
JNI_OnLoad (JavaVM * pVM, void * pReserved)
{
  TIMInit ();

  MEMInit ();
  NETInit ();
  PROInit ();

  return JNI_VERSION_1_2;
} /* JNI_OnLoad (JavaVM *, void *) */


/*******************************************************************\
| Memory Information                                                |
\*******************************************************************/

JNIEXPORT void JNICALL
Java_SysInformer_memGetApplicationMemory (JNIEnv * env, jclass cls, jlongArray cntrs)
{
  uint64 * pCntrs;
  assert (cntrs);

  pCntrs = (uint64 *) env->GetLongArrayElements (cntrs, NULL);
  MEMGetApplicationMemory (
        pCntrs + SysInformer_MIDX_MemoryResident,
        pCntrs + SysInformer_MIDX_MemorySwapped);
  env->ReleaseLongArrayElements (cntrs, (jlong *) pCntrs, 0);
} // memGetApplicationMemory (long [])


JNIEXPORT void JNICALL
Java_SysInformer_memGetKernelMemory (JNIEnv * env, jclass cls, jlongArray cntrs)
{
  uint64 * pCntrs;
  assert (cntrs);

  pCntrs = (uint64 *) env->GetLongArrayElements (cntrs, NULL);
  MEMGetKernelMemory (pCntrs + SysInformer_MIDX_MemoryUsed);
  env->ReleaseLongArrayElements (cntrs, (jlong *) pCntrs, 0);
} // memGetKernelMemory (long [])


JNIEXPORT void JNICALL
Java_SysInformer_memGetTotalMemory (JNIEnv * env, jclass cls, jlongArray cntrs)
{
  uint64 * pCntrs;
  assert (cntrs);

  pCntrs = (uint64 *) env->GetLongArrayElements (cntrs, NULL);
  MEMGetTotalMemory (
        pCntrs + SysInformer_MIDX_PhysicalUsed,
        pCntrs + SysInformer_MIDX_PhysicalFree,
        pCntrs + SysInformer_MIDX_SwapUsed);
  env->ReleaseLongArrayElements (cntrs, (jlong *) pCntrs, 0);
} // memGetTotalMemory (long [])


/*******************************************************************\
| Network Information                                               |
\*******************************************************************/

JNIEXPORT void JNICALL
Java_SysInformer_netGetTotalTraffic (JNIEnv * env, jclass cls, jlongArray cntrs)
{
  uint64 * pCntrs;
  assert (cntrs);

  pCntrs = (uint64 *) env->GetLongArrayElements (cntrs, NULL);
  NETGetTotalTraffic (
        pCntrs + SysInformer_NIDX_SentBytes,
        pCntrs + SysInformer_NIDX_ReceivedBytes,
        pCntrs + SysInformer_NIDX_SentPackets,
        pCntrs + SysInformer_NIDX_ReceivedPackets);
  env->ReleaseLongArrayElements (cntrs, (jlong *) pCntrs, 0);
} // netGetTotalTraffic (long [])


/*******************************************************************\
| Processor Information                                             |
\*******************************************************************/

JNIEXPORT void JNICALL
Java_SysInformer_proGetTotalUsage (JNIEnv * env, jclass cls, jlongArray cntrs)
{
  uint64 * pCntrs;
  assert (cntrs);

  pCntrs = (uint64 *) env->GetLongArrayElements (cntrs, NULL);
  PROGetTotalUsage (
        pCntrs + SysInformer_PIDX_KernelTicks,
        pCntrs + SysInformer_PIDX_WaitTicks,
        pCntrs + SysInformer_PIDX_UserTicks,
        pCntrs + SysInformer_PIDX_IdleTicks);
  env->ReleaseLongArrayElements (cntrs, (jlong *) pCntrs, 0);
} // proGetTotalUsage (long [])


/*******************************************************************\
| High Performance Timer                                            |
\*******************************************************************/

JNIEXPORT void JNICALL
Java_SysInformer_timGetConfig (JNIEnv * env, jclass cls, jlongArray cntrs)
{
  uint64 * pCntrs;
  assert (cntrs);

  pCntrs = (uint64 *) env->GetLongArrayElements (cntrs, NULL);
  pCntrs [SysInformer_TIDX_TimerScale] = iTIMTimerScale;
  pCntrs [SysInformer_TIDX_TimerGranularity] = iTIMTimerGranularity;
  env->ReleaseLongArrayElements (cntrs, (jlong *) pCntrs, 0);
} // timGetConfig (long [])


JNIEXPORT jlong JNICALL
Java_SysInformer_timGetTimestamp (JNIEnv * env, jclass cls)
{
  return (jlong) TIMGetTimestamp ();
} // timGetTimestamp ()
