/*******************************************************************\
| SuiteClient.java (Concurrency Client Connections)                 |
+-------------------------------------------------------------------+
| Copyright (C) Lubomir Bulej <lubomir.bulej@mff.cuni.cz>           |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

import org.omg.CORBA.IntHolder;
import org.omg.CORBA.SystemException;


/*******************************************************************\
| ThreadAction                                                      |
\*******************************************************************/

final class ThreadAction implements Runnable {

	private IntHolder thisClient;

	public ThreadAction (IntHolder thisClient) {
		this.thisClient = thisClient;
	} // ThreadAction (IntHolder)


	public final void run () {
		IntHolder theClient = new IntHolder ();

		try {
			do {
				SuiteAction.pingServer.Pong (theClient);
			} while (theClient.value != thisClient.value);

		} catch (SystemException ex) {
			BenchGlobal.displayFatal ("BEN", "Client "+ ex.toString ());
		}
	} // run ()

} // final class ThreadAction


/*******************************************************************\
| SuiteClient                                                       |
\*******************************************************************/

public final class SuiteClient {

	private static final int BEN_NumThreads = 10;


	public static String getName () {
		return ("Concurrency Client Connections Client");
	} // getName ()


	public static void run (org.omg.CORBA.Object suiteServer) {
		IntHolder thisClient	= new IntHolder ();
		IntHolder maxClients	= new IntHolder ();

		Thread [] threads 		= new Thread [BEN_NumThreads];
		ThreadAction action		= new ThreadAction (thisClient);


		try {
			/*
			 * Execute the measurement.
			 */
			SuiteAction.pingServer = PingHelper.narrow (suiteServer);
			SuiteAction.pingServer.Register (thisClient, maxClients);

			ActionStamp stamps [] = new ActionStamp [0];

			/*
			 * Create threads to provide the load on clients that
			 * will not be selected for the measurement itself.
			 */
			for (int tnum = 0; tnum < threads.length; tnum++) {
				threads [tnum] = new Thread (action);
				threads [tnum].start ();
			}

			/*
			 * The next phase of the benchmark starts when the
			 * threads finish. The threads will finish only when
			 * this client was selected for the measurement.
			 */
			for (int tnum = 0; tnum < threads.length; tnum++) {
				MeasureCore.joinThread (threads [tnum]);
			}

			MeasureCore.measure (
				BenchGlobal.DSP_FieldClients + maxClients.value +"\t"+
				BenchGlobal.DSP_FieldThreads + threads.length +"\t",
				stamps, threads.length, BenchGlobal.SAM_TestSamples);

		} catch (SystemException ex) {
			BenchGlobal.displayFatal ("BEN", "Client "+ ex.toString ());
		}
	} // run (String [])

} // public final class SuiteClient

