/*******************************************************************\
| SuiteClient.java (Dispatch Servant Instances)                     |
+-------------------------------------------------------------------+
| Copyright (C) Lubomir Bulej <lubomir.bulej@mff.cuni.cz>           |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

import org.omg.CORBA.IntHolder;
import org.omg.CORBA.SystemException;


/*******************************************************************\
| SuiteClient                                                       |
\*******************************************************************/

public final class SuiteClient {

	/**
	 * Benchmark configuration.
	 */
	private static final int BEN_MinInstances	= 1;
	private static final int BEN_MaxInstances	= BenchGlobal.SAM_MaxInstances;
	private static final int BEN_MinThreads		= 1;
	private static final int BEN_MaxThreads		= BenchGlobal.SAM_MaxThreads;
	private static final int BEN_StepLimit		= 5;
	private static final int BEN_StepScale 		= 10;


	public static String getName () {
		return ("Dispatch Servant Instances Client");
	} // getName ()


	public static void run (org.omg.CORBA.Object suiteServer) {

		try {
			/*
			 * Get the parameters.
			 */
			int threadsMin = BenchGlobal.getIntProperty ("jampler.threads.min", BEN_MinThreads);
			int threadsMax = BenchGlobal.getIntProperty ("jampler.threads.max", BEN_MaxThreads);
			int threadsLimit = BenchGlobal.getIntProperty ("jampler.threads.limit", BEN_StepLimit);
			int threadsScale = BenchGlobal.getIntProperty ("jampler.threads.scale", BEN_StepScale);

			int instancesMin = BenchGlobal.getIntProperty ("jampler.instances.min", BEN_MinInstances);
			int instancesMax = BenchGlobal.getIntProperty ("jampler.instances.max", BEN_MaxInstances);
			int instancesLimit = BenchGlobal.getIntProperty ("jampler.instances.limit", BEN_StepLimit);
			int instancesScale = BenchGlobal.getIntProperty ("jampler.instances.scale", BEN_StepScale);

			IntHolder instancesCount = new IntHolder ();
			IntHolder instancesStep = new IntHolder ();
			IntHolder threadsCount = new IntHolder ();
			IntHolder threadsStep = new IntHolder ();

			ActionStamp stamps [] = new ActionStamp [0];

			/*
			 * Execute the measurement.
			 */
			Ping [] pingSequenceAddition;
			PingFactory pingFactory = PingFactoryHelper.narrow (suiteServer);

			/*
			 * Originally, this was implemented as calling all objects from
			 * the sequence. For a large number of objects, however, this
			 * took too long time to complete.
			 *
			 * An alternative, applied here, is to call a single random
			 * object as an action. Provided that the scaling mechanism will
			 * call the action enough times, the results should be similar.
			 */
			SuiteAction.pingSequenceLength = 0;
			SuiteAction.pingSequence = new Ping [BenchGlobal.SAM_MaxInstances];

			MeasureCore.initialStep (
				instancesStep, instancesCount,
				instancesMin, instancesScale);

			while (instancesCount.value <= instancesMax) {
				/*
				 * Get additional objects for this step of the benchmark
				 * and append them at the end of the sequence holding all
				 * objects. Be sure to delete the sequence holding the
				 * additional objects at once so that it does not distort
				 * the measurement.
				 *
				 * Allocate instances in 10K chunks at most -- JacORB 1.4.1
				 * has problems returning more than ~21K instances at a time.
				 */
				int toCreate = instancesCount.value - SuiteAction.pingSequenceLength;
				while (toCreate > 0) {
					pingSequenceAddition = pingFactory.Create (toCreate > 10000 ? 10000 : toCreate);

					System.arraycopy (
						pingSequenceAddition, 0,
						SuiteAction.pingSequence, SuiteAction.pingSequenceLength,
						pingSequenceAddition.length);

					SuiteAction.pingSequenceLength += pingSequenceAddition.length;
					toCreate -= pingSequenceAddition.length;
					pingSequenceAddition = null;
				}

				/*
				 * Execute the measurement for all thread numbers.
				 */
				MeasureCore.initialStep (
					threadsStep, threadsCount,
					threadsMin, threadsScale);

				while (threadsCount.value <= threadsMax) {
					MeasureCore.measure (
						BenchGlobal.DSP_FieldInstances + instancesCount.value +"\t"+
						BenchGlobal.DSP_FieldThreads + threadsCount.value +"\t",
						stamps, threadsCount.value, BenchGlobal.SAM_TestSamples);

					MeasureCore.advanceStep (
						threadsStep, threadsCount,
						threadsLimit, threadsScale);
				} // threads loop

				/*
				 * There is no deallocation of servants.
				 */
				MeasureCore.advanceStep (
					instancesStep, instancesCount,
					instancesLimit, instancesScale);
			} // instances loop

		} catch (SystemException ex) {
			BenchGlobal.displayFatal ("BEN", "Client "+ ex.toString ());
		} catch (IllegalArgumentException ex) {
			BenchGlobal.displayFatal ("BEN", "Client "+ ex.getMessage ());
		}
	} // run (String [])

} // public final class SuiteClient

