/*******************************************************************\
| FactServant.java (Dispatch Servant Lifecycle)                     |
+-------------------------------------------------------------------+
| Copyright (C) Lubomir Bulej <lubomir.bulej@mff.cuni.cz>           |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

import org.omg.CORBA.LongHolder;
import org.omg.CORBA.UserException;
import org.omg.CORBA.SystemException;


/*******************************************************************\
| FactServant                                                       |
\*******************************************************************/

public final class FactServant extends EmptyFactoryPOA {

	/**
	 * Creates an empty servant instance and
	 * returns the amount of time it took.
	 */
	public final void CreateIndividual (
		EmptyHolder oEmpty, LongHolder iTimeInvoke, LongHolder iTimeReturn) {

		iTimeInvoke.value = SysInformer.timGetTimestamp ();

		try {
			EmptyServant emptyServant = new EmptyServant ();
			oEmpty.value = EmptyHelper.narrow (
					BenchBroker.rootPOA.servant_to_reference (emptyServant));

		} catch (SystemException ex) {
			BenchGlobal.displayFatal ("BEN", "Server "+ ex.toString ());
		} catch (UserException ex) {
			BenchGlobal.displayFatal ("BEN", "Server "+ ex.toString ());
		}

		iTimeReturn.value = SysInformer.timGetTimestamp ();
	} // CreateIndividual (EmptyHolder, LongHolder, LongHolder)


	/**
	 * Destroys an empty servant instance and
	 * returns the amount of time it took.
	 */
	public final void DeleteIndividual (
		Empty oEmpty, LongHolder iTimeInvoke, LongHolder iTimeReturn) {

		iTimeInvoke.value = SysInformer.timGetTimestamp ();

		try {
			byte [] emptyOID = BenchBroker.rootPOA.reference_to_id (oEmpty);
			BenchBroker.rootPOA.deactivate_object (emptyOID);

		} catch (SystemException ex) {
			BenchGlobal.displayFatal ("BEN", "Server "+ ex.toString ());
		} catch (UserException ex) {
			BenchGlobal.displayFatal ("BEN", "Server "+ ex.toString ());
		}

		iTimeReturn.value = SysInformer.timGetTimestamp ();
	} // DeleteIndividual (Empty, LongHolder, LongHolder)


	/**
	 * Creates a specified amount of empty servants and
	 * returns a sequence of object references to them.
	 */
	public final Empty [] CreateSequence (int iLength) {
		Empty [] emptySequence = null;

		try {
			EmptyServant emptyServant;
			emptySequence = new Empty [iLength];

			for (int count = 0; count < iLength; count++) {
				emptyServant = new EmptyServant ();
				emptySequence [count] = EmptyHelper.narrow (
					BenchBroker.rootPOA.servant_to_reference (emptyServant));
			}

		} catch (SystemException ex) {
			BenchGlobal.displayFatal ("BEN", "Server "+ ex.toString ());
		} catch (UserException ex) {
			BenchGlobal.displayFatal ("BEN", "Server "+ ex.toString ());
		}

		return emptySequence;
	} // CreateSequence (int)

} // public final class FactServant

