/*******************************************************************\
| SuiteAction.java (Dispatch Servant Lifecycle)                     |
+-------------------------------------------------------------------+
| Copyright (C) Lubomir Bulej <lubomir.bulej@mff.cuni.cz>           |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

import org.omg.CORBA.LongHolder;


/*******************************************************************\
| SuiteAction                                                       |
\*******************************************************************/

public final class SuiteAction {

	private static final int BEN_StampCreateInvoke	= 0;
	private static final int BEN_StampCreateReturn	= 1;
	private static final int BEN_StampCreateFinish	= 2;
	private static final int BEN_StampDeleteInvoke	= 3;
	private static final int BEN_StampDeleteReturn	= 4;
	private static final int BEN_StampDeleteFinish	= 5;

	/**
	 * Suite server object.
	 */
	public static int			emptySequenceLength;
	public static Empty []		emptySequence;
	public static EmptyFactory	emptyFactory;

	/**
	 * Holder required by the execute() method. These have to be instance
	 * variables to avoid repeated allocations/deallocations upon invocation
	 * of the execute() method. Moreover, the variables are thread-safe
	 * because every measurement thread has its own instance of SuiteAction.
	 */
	private LongHolder			invokeStamp = new LongHolder ();
	private LongHolder			returnStamp = new LongHolder ();
	private EmptyHolder			emptyHolder = new EmptyHolder ();

	/**
	 * Calls the EmptyFactory to create and later delete an Empty
	 * servant instance. It measures the time it takes to create
	 * or delete the object on the server and the time needed to
	 * create or delete proxy on the client.
	 */
	public final void execute (long [] times) {
		emptyFactory.CreateIndividual (emptyHolder, invokeStamp, returnStamp);
		times [BEN_StampCreateFinish] = SysInformer.timGetTimestamp ();
		times [BEN_StampCreateInvoke] = invokeStamp.value;
		times [BEN_StampCreateReturn] = returnStamp.value;

		emptyFactory.DeleteIndividual (emptyHolder.value, invokeStamp, returnStamp);
		times [BEN_StampDeleteFinish] = SysInformer.timGetTimestamp ();
		times [BEN_StampDeleteInvoke] = invokeStamp.value;
		times [BEN_StampDeleteReturn] = returnStamp.value;

		times [BEN_StampDeleteInvoke] -= times [BEN_StampCreateFinish];
		times [BEN_StampDeleteReturn] -= times [BEN_StampCreateFinish];
		times [BEN_StampDeleteFinish] -= times [BEN_StampCreateFinish];

		emptyHolder.value = null;
	} // execute ()

} // public final class SuiteAction
