/*******************************************************************\
| SuiteClient.java (Dispatch Servant Lifecycle)                     |
+-------------------------------------------------------------------+
| Copyright (C) Lubomir Bulej <lubomir.bulej@mff.cuni.cz>           |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

import org.omg.CORBA.IntHolder;
import org.omg.CORBA.SystemException;


/*******************************************************************\
| SuiteClient                                                       |
\*******************************************************************/

public final class SuiteClient {

	/**
	 * Benchmark configuration.
	 */
	private static final int BEN_MinInstances	= 1;
	private static final int BEN_MaxInstances	= BenchGlobal.SAM_MaxInstances;
	private static final int BEN_MinThreads		= 1;
	private static final int BEN_MaxThreads		= BenchGlobal.SAM_MaxThreads;
	private static final int BEN_StepLimit		= 5;
	private static final int BEN_StepScale 		= 10;


	public static String getName () {
		return ("Dispatch Servant Lifecycle Client");
	} // getName ()


	public static void run (org.omg.CORBA.Object suiteServer) {

		try {
			/*
			 * Get the parameters.
			 */
			int threadsMin = BenchGlobal.getIntProperty ("jampler.threads.min", BEN_MinThreads);
			int threadsMax = BenchGlobal.getIntProperty ("jampler.threads.max", BEN_MaxThreads);
			int threadsLimit = BenchGlobal.getIntProperty ("jampler.threads.limit", BEN_StepLimit);
			int threadsScale = BenchGlobal.getIntProperty ("jampler.threads.scale", BEN_StepScale);

			int instancesMin = BenchGlobal.getIntProperty ("jampler.instances.min", BEN_MinInstances);
			int instancesMax = BenchGlobal.getIntProperty ("jampler.instances.max", BEN_MaxInstances);
			int instancesLimit = BenchGlobal.getIntProperty ("jampler.instances.limit", BEN_StepLimit);
			int instancesScale = BenchGlobal.getIntProperty ("jampler.instances.scale", BEN_StepScale);

			IntHolder instancesCount = new IntHolder ();
			IntHolder instancesStep = new IntHolder ();
			IntHolder threadsCount = new IntHolder ();
			IntHolder threadsStep = new IntHolder ();

			ActionStamp stamps [] = new ActionStamp [6];
			stamps [0] = new ActionStamp ("CreateInvoke",
				ActionStamp.ASF_Remote | ActionStamp.ASF_Absolute);
			stamps [1] = new ActionStamp ("CreateReturn",
				ActionStamp.ASF_Remote | ActionStamp.ASF_Absolute);
			stamps [2] = new ActionStamp ("CreateFinish",
				ActionStamp.ASF_Local | ActionStamp.ASF_Absolute);
			stamps [3] = new ActionStamp ("DeleteInvoke",
				ActionStamp.ASF_Remote);
			stamps [4] = new ActionStamp ("DeleteReturn",
				ActionStamp.ASF_Remote);
			stamps [5] = new ActionStamp ("DeleteFinish",
				ActionStamp.ASF_Local);

			/*
			 * Execute the measurement.
			 */
			SuiteAction.emptyFactory = EmptyFactoryHelper.narrow (suiteServer);
			SuiteAction.emptySequence = new Empty [BenchGlobal.SAM_MaxInstances];
			SuiteAction.emptySequenceLength = 0;

			MeasureCore.initialStep (
				instancesStep, instancesCount,
				instancesMin, instancesScale);

			while (instancesCount.value <= instancesMax) {
				int 		toCreate;
				Empty [] 	emptySequenceAddition;

				/*
				 * Get additional objects for this step of the benchmark
				 * and append them at the end of the sequence holding all
				 * objects. Be sure to delete the sequence holding the
				 * additional objects at once so that it does not distort
				 * the measurement.
				 *
				 * Contrary to C++ version, we have to keep the returned
				 * objects around so as to prevent the garbage collector
				 * from cleaning them up.
				 *
				 * Allocate instances in 10K chunks at most -- JacORB 1.4.1
				 * has problems returning more than ~21K instances at a time.
				 */
				toCreate = instancesCount.value - SuiteAction.emptySequenceLength;
				while (toCreate > 0) {
					emptySequenceAddition =
						SuiteAction.emptyFactory.CreateSequence (
							toCreate > 10000 ? 10000 : toCreate);

					System.arraycopy (
						emptySequenceAddition, 0,
						SuiteAction.emptySequence, SuiteAction.emptySequenceLength,
						emptySequenceAddition.length);

					SuiteAction.emptySequenceLength += emptySequenceAddition.length;
					toCreate -= emptySequenceAddition.length;
					emptySequenceAddition = null;
				}

				/*
				 * Execute the measurement for all thread numbers.
				 */
				MeasureCore.initialStep (
					threadsStep, threadsCount,
					threadsMin, threadsScale);

				while (threadsCount.value <= threadsMax) {
					MeasureCore.measure (
						BenchGlobal.DSP_FieldInstances + instancesCount.value +"\t"+
						BenchGlobal.DSP_FieldThreads + threadsCount.value +"\t",
						stamps, threadsCount.value, BenchGlobal.SAM_TestSamples);

					MeasureCore.advanceStep (
						threadsStep, threadsCount,
						threadsLimit, threadsScale);
				} // threads loop

				/*
				 * There is no deallocation of servants.
				 */
				MeasureCore.advanceStep (
					instancesStep, instancesCount,
					instancesLimit, instancesScale);
			} // instances loop

		} catch (SystemException ex) {
			BenchGlobal.displayFatal ("BEN", "Client "+ ex.toString ());
		} catch (IllegalArgumentException ex) {
			BenchGlobal.displayFatal ("BEN", "Client "+ ex.getMessage ());
		}
	} // run (String [])

} // public final class SuiteClient

