/*******************************************************************\
| SuiteAction.java (Invocation Initial)                             |
+-------------------------------------------------------------------+
| Copyright (C) Lubomir Bulej <lubomir.bulej@mff.cuni.cz>           |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

import org.omg.CORBA.LongHolder;


/*******************************************************************\
| SuiteAction                                                       |
\*******************************************************************/

public final class SuiteAction {

	private static final int BEN_StampRoundtrip1st	= 0;
	private static final int BEN_StampRoundtrip2nd	= 1;
	private static final int BEN_StampRoundtrip3rd	= 2;

	/**
	 * Suite server object.
	 */
	public static PingFactory	pingFactory;

	/**
	 * Holder required by the execute() method. These have to be instance
	 * variables to avoid repeated allocations/deallocations upon invocation
	 * of the execute() method. Moreover, the variables are thread-safe
	 * because every measurement thread has its own instance of SuiteAction.
	 */
	private PingHolder			pingHolder = new PingHolder ();

	/**
	 * Asks the PingFactory to create a Ping Servant and invokes the
	 * Pong method on it three times. It measures the time needed for
	 * each of the invocations.
	 */
	public final void execute (long [] times) {
		long timeBefore;
		pingFactory.CreateIndividual (pingHolder);

		timeBefore = SysInformer.timGetTimestamp ();

		pingHolder.value.Pong ();
		times [BEN_StampRoundtrip1st] = SysInformer.timGetTimestamp ();
		pingHolder.value.Pong ();
		times [BEN_StampRoundtrip2nd] = SysInformer.timGetTimestamp ();
		pingHolder.value.Pong ();
		times [BEN_StampRoundtrip3rd] = SysInformer.timGetTimestamp ();

		times [BEN_StampRoundtrip3rd] -= times [BEN_StampRoundtrip2nd];
		times [BEN_StampRoundtrip2nd] -= times [BEN_StampRoundtrip1st];
		times [BEN_StampRoundtrip1st] -= timeBefore;

		pingFactory.DeleteIndividual (pingHolder.value);
		pingHolder.value = null;
	} // execute ()

} // public final class SuiteAction
