/*******************************************************************\
| SuiteClient.java (Marshal Octet Sequence)                         |
+-------------------------------------------------------------------+
| Copyright (C) Lubomir Bulej <lubomir.bulej@mff.cuni.cz>           |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

import org.omg.CORBA.IntHolder;
import org.omg.CORBA.SystemException;


/*******************************************************************\
| SuiteClient                                                       |
\*******************************************************************/

public final class SuiteClient {

	/**
	 * Benchmark configuration.
	 */
	private static final int BEN_MinOctets		= 1;
	private static final int BEN_MaxOctets		= BenchGlobal.SAM_MaxOctets;
	private static final int BEN_MinThreads		= 1;
	private static final int BEN_MaxThreads		= BenchGlobal.SAM_MaxThreads;
	private static final int BEN_StepLimit		= 5;
	private static final int BEN_StepScale 		= 10;


	public static String getName () {
		return ("Marshal Octet Sequence Client");
	} // getName ()


	public static void run (org.omg.CORBA.Object suiteServer) {

		try {
			/*
			 * Get the parameters.
			 */
			int threadsMin = BenchGlobal.getIntProperty ("jampler.threads.min", BEN_MinThreads);
			int threadsMax = BenchGlobal.getIntProperty ("jampler.threads.max", BEN_MaxThreads);
			int threadsLimit = BenchGlobal.getIntProperty ("jampler.threads.limit", BEN_StepLimit);
			int threadsScale = BenchGlobal.getIntProperty ("jampler.threads.scale", BEN_StepScale);

			int octetsMin = BenchGlobal.getIntProperty ("jampler.octets.min", BEN_MinOctets);
			int octetsMax = BenchGlobal.getIntProperty ("jampler.octets.max", BEN_MaxOctets);
			int octetsLimit = BenchGlobal.getIntProperty ("jampler.octets.limit", BEN_StepLimit);
			int octetsScale = BenchGlobal.getIntProperty ("jampler.octets.scale", BEN_StepScale);

			String argMode = BenchGlobal.getArgMode ();

			IntHolder threadsCount = new IntHolder ();
			IntHolder threadsStep = new IntHolder ();
			IntHolder octetsCount = new IntHolder ();
			IntHolder octetsStep = new IntHolder ();

			ActionStamp stamps [] = new ActionStamp [0];

			/*
			 * Execute the measurement.
			 */
			SuiteAction.pingServer = PingHelper.narrow (suiteServer);

			MeasureCore.initialStep (
				octetsStep, octetsCount,
				octetsMin, octetsScale);

			while (octetsCount.value <= octetsMax) {
				/*
				 * Set the length of the sequence to be
				 * allocated upon SuiteAction instantiation.
				 */
				SuiteAction.sequenceLength = octetsCount.value;

				/*
				 * Execute the measurement for all thread numbers.
				 */
				MeasureCore.initialStep (
					threadsStep, threadsCount,
					threadsMin, threadsScale);

				while (threadsCount.value <= threadsMax) {
					MeasureCore.measure (
						BenchGlobal.DSP_FieldDirection + argMode +"\t"+
						BenchGlobal.DSP_FieldOctets + octetsCount.value +"\t"+
						BenchGlobal.DSP_FieldThreads + threadsCount.value +"\t",
						stamps, threadsCount.value, BenchGlobal.SAM_TestSamples);

					MeasureCore.advanceStep (
						threadsStep, threadsCount,
						threadsLimit, threadsScale);
				} // threads loop

				MeasureCore.advanceStep (
					octetsStep, octetsCount,
					octetsLimit, octetsScale);
			} // octets loop

		} catch (SystemException ex) {
			BenchGlobal.displayFatal ("BEN", "Client "+ ex.toString ());
		} catch (IllegalArgumentException ex) {
			BenchGlobal.displayFatal ("BEN", "Client "+ ex.getMessage ());
		}
	} // run (String [])

} // public final class SuiteClient

