#--------------------------------------------------------------------
# Targets
#--------------------------------------------------------------------

IDL_INTO := Server
IDL_FROM := ../../../../C++/_Ampler/idl_main.idl
include $(MAK_MAKE_IDL)
ALL_JAVA += $(NEW_JAVA)

#--------------------------------------------------------------------
# Files
#--------------------------------------------------------------------

BEN_JAVA :=	BenchClient BenchServer BenchGlobal BenchBroker \
		PeerLocator LongSampler MeasureCore ActionStamp \
		SysInformer
ALL_JAVA +=	$(addprefix ../../../_Ampler/,$(BEN_JAVA))

ALL_SRCS :=	$(addsuffix .java,$(ALL_JAVA))


#--------------------------------------------------------------------

UTL_JAVA :=	FileExpander FileStripper 
UTL_SRCS :=	$(addprefix ../../../_Jtools/,$(addsuffix .java,$(UTL_JAVA)))

#--------------------------------------------------------------------

SHR_DEFS +=	../../../all_defs.h ../../../shr_defs.h \
		com_defs.h plt_defs.h \
		$(if $(strip $(THR_BASE)), thr_defs.h)

LIB_OBJS :=	$(MEM_BASE) $(NET_BASE) $(PRO_BASE) \
		$(COM_BASE) $(PLT_BASE) $(TIM_BASE) \
		$(if $(strip $(THR_BASE)), $(THR_BASE))

#--------------------------------------------------------------------

JAR_FILE :=	$(MAK_DIR)Suite.jar

UTL_FILE :=	Jtools.jar

#--------------------------------------------------------------------
# Targets
#--------------------------------------------------------------------

$(UTL_FILE): $(UTL_SRCS)
	$(JC) -d . $(UTL_SRCS)
	$(JA) -cf $@ *.class
	$(call RM, *.class)

$(JAR_FILE): $(ALL_SRCS)
	$(JC) -d . -classpath $(JC_DIRS) $(ALL_SRCS)
	$(JA) -cf $@ *.class
	$(call RM,*.class)

DLL_FROM := SysInformer $(LIB_OBJS)
DLL_INTO := $(MAK_DIR)SysInformer
include $(MAK_MAKE_DLL)

#--------------------------------------------------------------------
# Targets
#--------------------------------------------------------------------

OBJ_FROM := ../../../_Ampler/SysInformer.cc
OBJ_DEFS := $(SHR_DEFS) ../../../_Ampler/SysInformer.h
include $(MAK_MAKE_OBJ)

#--------------------------------------------------------------------

OBJ_FROM := ../../../../C++/_Ported/InfMemory/mem_%.cc
OBJ_DEFS := $(SHR_DEFS)
include $(MAK_MAKE_OBJ)

OBJ_FROM := ../../../../C++/_Ported/InfNetwork/net_%.cc
OBJ_DEFS := $(SHR_DEFS)
include $(MAK_MAKE_OBJ)

OBJ_FROM := ../../../../C++/_Ported/InfProcessor/pro_%.cc
OBJ_DEFS := $(SHR_DEFS)
include $(MAK_MAKE_OBJ)

OBJ_FROM := ../../../../C++/_Ported/LibCompiler/com_%.cc
OBJ_DEFS := $(SHR_DEFS)
include $(MAK_MAKE_OBJ)

OBJ_FROM := ../../../../C++/_Ported/LibPlatform/plt_%.cc
OBJ_DEFS := $(SHR_DEFS)
include $(MAK_MAKE_OBJ)

OBJ_FROM := ../../../../C++/_Ported/LibThread/thr_%.cc
OBJ_DEFS := $(SHR_DEFS)
include $(MAK_MAKE_OBJ)

OBJ_FROM := ../../../../C++/_Ported/LibTimer/tim_%.cc
OBJ_DEFS := $(SHR_DEFS)
include $(MAK_MAKE_OBJ)

#--------------------------------------------------------------------

REM_LIST += com_defs.h
com_defs.h: ../../../../C++/_Ported/LibCompiler/$(COM_BASE).h
	$(call CP, $< $@) 

REM_LIST += plt_defs.h
plt_defs.h: ../../../../C++/_Ported/LibPlatform/$(PLT_BASE).h
	$(call CP, $< $@)
	
ifneq ($(strip $(THR_BASE)),"")
REM_LIST += thr_defs.h
thr_defs.h: ../../../../C++/_Ported/LibThread/$(THR_BASE).h
	$(call CP, $< $@)
endif

#--------------------------------------------------------------------

JT_FEXP := $(JR) -classpath $(UTL_FILE) FileExpander
JT_FSTR := $(JR) -classpath $(UTL_FILE) FileStripper

$(CLT_FILE): $(CLT_BASE).tpl $(UTL_FILE)
	$(JT_FEXP) $(CLT_BASE).tpl JR_FILE="$(notdir $@)" \
	 	JR="$(JR)" JR_DIRS="$(JR_DIRS)" \
	 	JR_ORBC="$(JR_ORBC)" JR_ORBS="$(JR_ORBS)" JR_DEFS="$(JR_DEFS)" \
	 	JR_MODE="$(SUI_DIRS)" JR_TYPE="$(SUI_TYPE)" JR_SIZE="$(SUI_SIZE)" \
	 	JR_MAIN="BenchClient" JR_ARGS="$(JR_ARGS)" > $@
	$(call CM,$@)

$(SRV_FILE): $(SRV_BASE).tpl $(UTL_FILE)
	$(JT_FEXP) $(SRV_BASE).tpl JR_FILE="$(notdir $@)" \
	 	JR="$(JR)" JR_DIRS="$(JR_DIRS)" \
	 	JR_ORBC="$(JR_ORBC)" JR_ORBS="$(JR_ORBS)" JR_DEFS="$(JR_DEFS)" \
	 	JR_MAIN="BenchServer" JR_ARGS="$(JR_ARGS)" > $@
	$(call CM,$@)

#--------------------------------------------------------------------
# Shared Targets
#--------------------------------------------------------------------

shared_all:     $(DLL_LIST) $(UTL_FILE) $(JAR_FILE) $(CLT_FILE) $(SRV_FILE)

shared_clean:
	-$(call RM,Jampler.IOR $(wildcard *.class $(REM_LIST) $(UTL_FILE)))

shared_cleanall: shared_clean
	-$(call RM,$(JAR_FILE) $(wildcard $(CLT_FILE) $(SRV_FILE) $(DLL_LIST)))

#--------------------------------------------------------------------

debug:
	@$(MAKE) "DEBUG=YES"

#--------------------------------------------------------------------
