/*******************************************************************\
| SHR_Defs.H                                                        |
+-------------------------------------------------------------------+
| Definitions for all shared modules.                               |
+-------------------------------------------------------------------+
| Copyright (C) Petr Tuma <petr.tuma@mff.cuni.cz>                   |
|               and other contributing authors of                   |
|               Distributed Systems Research Group                  |
|               Department of Software Engineering                  |
|               Faculty of Mathematics and Physics                  |
|               Charles University, Czech Republic                  |
\*******************************************************************/

//-------------------------------------------------------------------

/*******************************************************************\
| Basics                                                            |
\*******************************************************************/

#define FALSE             0
#define TRUE              1

//-------------------------------------------------------------------

/*******************************************************************\
| Configuration                                                     |
\*******************************************************************/

// Macros to handle different versions of the stream library

#if   defined DSPStreamsOLD
  #define DSPIStream    istream
  #define DSPOStream    ostream
  #define DSPIOStream   iostream
  #define DSPEndLine    endl
  #define DSPFlush      flush
  #define DSPStdOut     cout
  #define DSPStdErr     cerr
#elif defined DSPStreamsNEW
  #define DSPIStream    std::istream
  #define DSPOStream    std::ostream
  #define DSPIOStream   std::iostream
  #define DSPEndLine    std::endl
  #define DSPFlush      std::flush
  #define DSPStdOut     std::cout
  #define DSPStdErr     std::cerr
#else
  #error
#endif

//-------------------------------------------------------------------

/*******************************************************************\
| Platform Dependent Information Modules                            |
\*******************************************************************/

/*******************************************************************\
| MEM_*.CC Definitions                                              |
\*******************************************************************/

void MEMInit (void);
void MEMGetApplicationMemory (uint64 *pMemoryResident, uint64 *pMemorySwapped);
void MEMGetKernelMemory (uint64 *pMemoryUsed);
void MEMGetTotalMemory (uint64 *pPhysicalUsed, uint64 *pPhysicalFree, uint64 *pSwapUsed);

/*******************************************************************\
| NET_*.CC Definitions                                              |
\*******************************************************************/

void NETInit (void);
void NETGetTotalTraffic (uint64 *pSentBytes, uint64 *pReceivedBytes, uint64 *pSentPackets, uint64 *pReceivedPackets);

/*******************************************************************\
| PRO_*.CC Definitions                                              |
\*******************************************************************/

void PROInit (void);
void PROGetApplicationThreads (uint64 *pThreads);
void PROGetTotalUsage (uint64 *pKernelTicks, uint64 *pWaitTicks, uint64 *pUserTicks, uint64 *pIdleTicks);

//-------------------------------------------------------------------

/*******************************************************************\
| Platform Dependent Library Modules                                |
\*******************************************************************/

/*******************************************************************\
| THR_*.CC Definitions                                              |
\*******************************************************************/

void      THRThreadCreate  (THRThread *pThread, void *(*pFunction) (void *), void *pArgument);
void      THRThreadFinish  (THRThread *pThread);
void      THRThreadSleep   (int iSeconds);

void      THRLockCreate    (THRLock *pLock);
void      THRLockFinish    (THRLock *pLock);
void      THRLockLock      (THRLock *pLock);
void      THRLockUnlock    (THRLock *pLock);

/*******************************************************************\
| TIM_*.CC Definitions                                              |
\*******************************************************************/

extern uint64 iTIMTimerScale;
extern uint64 iTIMTimerGranularity;

void   TIMInit         (void);
uint64 TIMGetTimestamp (void);

//-------------------------------------------------------------------
